@@ -16,21 +16,23 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
-import sys, os
+
+import sys, os, time
 import GNS3.Globals as globals
 import GNS3.Config.Defaults as Defaults
-from PyQt4.QtGui import QApplication
-from PyQt4.QtCore import QVariant, QSettings
-from GNS3.Utils import Singleton
+from PyQt4.QtGui import QApplication, QSplashScreen, QPixmap
+from PyQt4.QtCore import Qt, QVariant, QSettings, QEventLoop
+from GNS3.Utils import Singleton, translate
 from GNS3.Workspace import Workspace
-from GNS3.Config.Objects import systemDynamipsConf, systemGeneralConf, systemCaptureConf, systemQemuConf
+from GNS3.Config.Objects import systemDynamipsConf, systemGeneralConf, systemCaptureConf, systemQemuConf, systemVBoxConf
 from GNS3.Globals.Symbols import SYMBOLS, SYMBOL_TYPES
 from GNS3.Config.Config import ConfDB, GNS_Conf
 from GNS3.HypervisorManager import HypervisorManager
 from GNS3.QemuManager import QemuManager
+from GNS3.VBoxManager import VBoxManager
 from GNS3.Translations import Translator
 from GNS3.DynagenSub import DynagenSub
 from GNS3.ProjectDialog import ProjectDialog
@@ -57,6 +59,7 @@ class Application(QApplication, Singleton):
         self.__dynagen = None
         self.__HypervisorManager = None
         self.__QemuManager = None
+        self.__VBoxManager = None
 
         # Dict for storing config
         self.__systconf = {}
@@ -65,13 +68,16 @@ class Application(QApplication, Singleton):
         self.__hypervisors = {}
         self.__libraries = {}
         self.__qemuimages = {}
+        self.__vboximages = {}
         self.__piximages = {}
         self.__junosimages = {}
         self.__asaimages = {}
         self.__idsimages = {}
+        self.__recentfiles = []
         self.iosimages_ids = 0
         self.hypervisors_ids = 0
         self.qemuimages_ids = 0
+        self.vboximages_ids = 0
         self.piximages_ids = 0
         self.junosimages_ids = 0
         self.asaimages_ids = 0
@@ -171,7 +177,7 @@ class Application(QApplication, Singleton):
         return self.__iosimages
 
     iosimages = property(__getIOSImages, __setIOSImages, doc = 'IOS images dictionnary')
-    
+
     def __setQemuImages(self, qemuimages):
         """ register the sysconf instance
         """
@@ -185,7 +191,21 @@ class Application(QApplication, Singleton):
         return self.__qemuimages
 
     qemuimages = property(__getQemuImages, __setQemuImages, doc = 'Qemu images dictionnary')
-    
+
+    def __setVBoxImages(self, vboximages):
+        """ register the sysconf instance
+        """
+
+        self.__vboximages = vboximages
+
+    def __getVBoxImages(self):
+        """ return the sysconf instance
+        """
+
+        return self.__vboximages
+
+    vboximages = property(__getVBoxImages, __setVBoxImages, doc = 'VBox images dictionnary')
+
     def __setPIXImages(self, piximages):
         """ register the sysconf instance
         """
@@ -199,7 +219,7 @@ class Application(QApplication, Singleton):
         return self.__piximages
 
     piximages = property(__getPIXImages, __setPIXImages, doc = 'PIX images dictionnary')
-    
+
     def __setJunOSImages(self, junosimages):
         """ register the sysconf instance
         """
@@ -213,7 +233,7 @@ class Application(QApplication, Singleton):
         return self.__junosimages
 
     junosimages = property(__getJunOSImages, __setJunOSImages, doc = 'JunOS images dictionnary')
-    
+
     def __setASAImages(self, asaimages):
         """ register the sysconf instance
         """
@@ -227,7 +247,7 @@ class Application(QApplication, Singleton):
         return self.__asaimages
 
     asaimages = property(__getASAImages, __setASAImages, doc = 'ASA images dictionnary')
-    
+
     def __setIDSImages(self, idsimages):
         """ register the sysconf instance
         """
@@ -241,7 +261,7 @@ class Application(QApplication, Singleton):
         return self.__idsimages
 
     idsimages = property(__getIDSImages, __setIDSImages, doc = 'IDS images dictionnary')
-    
+
     def __setLibraries(self, libraries):
         """ register the sysconf instance
         """
@@ -256,6 +276,21 @@ class Application(QApplication, Singleton):
 
     libraries = property(__getLibraries, __setLibraries, doc = 'Libraries dictionnary')
 
+    def __setRecentFiles(self, recentfiles):
+        """ register the sysconf instance
+        """
+
+        self.__recentfiles = recentfiles
+
+    def __getRecentFiles(self):
+        """ return the sysconf instance
+        """
+
+        return self.__recentfiles
+
+    recentfiles = property(__getRecentFiles, __setRecentFiles, doc = 'Recent files array')
+
+
     def __setHypervisors(self, hypervisors):
         """ register the sysconf instance
         """
@@ -312,24 +347,61 @@ class Application(QApplication, Singleton):
 
     QemuManager = property(__getQemuManager, __setQemuManager, doc = 'QemuManager instance')
 
+    def __setVBoxManager(self, VBoxManager):
+        """ register the VBoxManager instance
+        """
+
+        self.__VBoxManager = VBoxManager
+
+    def __getVBoxManager(self):
+        """ return the VBoxManager instance
+        """
+
+        return self.__VBoxManager
+
+    VBoxManager = property(__getVBoxManager, __setVBoxManager, doc = 'VBoxManager instance')
+
+
+    def processSplashScreen(self):
+        """ Processes the splash screen, Prints a loading picture before entering the application
+		"""
+
+        self.splashMessage = translate("Application", "Starting Graphical Network Simulator...")
+        self.splashSleepTime = 1
+        self.splashPath = ':/images/logo_gns3_splash.png'
+
+        pixmap = QPixmap(self.splashPath)
+        self.splash = QSplashScreen(pixmap, Qt.WindowStaysOnTopHint)
+        self.splash.show()
+        self.splash.showMessage(self.splashMessage, Qt.AlignRight | Qt.AlignTop, Qt.black)
+
+        # make sure Qt really display the splash screen and message
+        self.processEvents(QEventLoop.AllEvents, 500)
+        time.sleep(self.splashSleepTime)
+        self.splash.finish(self.__mainWindow)
+        return
+
     def run(self, file):
 
+        # Display splash screen while waiting for the application to open
+        self.processSplashScreen()
+
         # Instantiation of Dynagen
         self.__dynagen = DynagenSub()
 
         self.systconf['dynamips'] = systemDynamipsConf()
         confo = self.systconf['dynamips']
-        confo.path = ConfDB().get('Dynamips/hypervisor_path', unicode('/usr/local/bin/dynamips'))
+        confo.path = ConfDB().get('Dynamips/hypervisor_path', Defaults.DYNAMIPS_DEFAULT_PATH)
         confo.port = int(ConfDB().get('Dynamips/hypervisor_port', 7200))
         confo.baseUDP = int(ConfDB().get('Dynamips/hypervisor_baseUDP', 10000))
-        confo.baseConsole = int(ConfDB().get('Dynamips/hypervisor_baseConsole', 2000))
-        confo.baseAUX = int(ConfDB().get('Dynamips/hypervisor_baseAUX', 0))
-        confo.workdir = ConfDB().get('Dynamips/hypervisor_working_directory', unicode(''))
+        confo.baseConsole = int(ConfDB().get('Dynamips/hypervisor_baseConsole', 2001))
+        confo.baseAUX = int(ConfDB().get('Dynamips/hypervisor_baseAUX', 2501))
+        confo.workdir = ConfDB().get('Dynamips/hypervisor_working_directory', Defaults.DYNAMIPS_DEFAULT_WORKDIR)
         confo.clean_workdir = ConfDB().value("Dynamips/clean_working_directory", QVariant(True)).toBool()
         confo.ghosting = ConfDB().value("Dynamips/dynamips_ghosting", QVariant(True)).toBool()
         confo.mmap = ConfDB().value("Dynamips/dynamips_mmap", QVariant(True)).toBool()
-        confo.sparsemem = ConfDB().value("Dynamips/dynamips_sparsemem", QVariant(False)).toBool()
-        confo.jitsharing = ConfDB().value("Dynamips/dynamips_jitsharing", QVariant(True)).toBool()
+        confo.sparsemem = ConfDB().value("Dynamips/dynamips_sparsemem", QVariant(True)).toBool()
+        confo.jitsharing = ConfDB().value("Dynamips/dynamips_jitsharing", QVariant(False)).toBool()
         if sys.platform.startswith('win'):
             confo.memory_limit = int(ConfDB().get("Dynamips/hypervisor_memory_usage_limit", 512))
         else:
@@ -337,6 +409,7 @@ class Application(QApplication, Singleton):
         confo.udp_incrementation = int(ConfDB().get("Dynamips/hypervisor_udp_incrementation", 100))
         confo.import_use_HypervisorManager = ConfDB().value("Dynamips/hypervisor_manager_import", QVariant(True)).toBool()
         confo.HypervisorManager_binding = ConfDB().get('Dynamips/hypervisor_manager_binding', unicode('127.0.0.1'))
+        confo.allocateHypervisorPerIOS = ConfDB().value("Dynamips/allocate_hypervisor_per_IOS", QVariant(True)).toBool()
 
         # replace ~user and $HOME by home directory
         if os.environ.has_key("HOME"):
@@ -348,33 +421,60 @@ class Application(QApplication, Singleton):
         # Qemu config
         self.systconf['qemu'] = systemQemuConf()
         confo = self.systconf['qemu']
-        confo.qemuwrapper_path = ConfDB().get('Qemu/qemuwrapper_path', unicode('/usr/local/libexec/gns3/qemuwrapper.py'))
-        confo.qemuwrapper_workdir = ConfDB().get('Qemu/qemuwrapper_working_directory', unicode(''))
+        confo.qemuwrapper_path = ConfDB().get('Qemu/qemuwrapper_path', Defaults.QEMUWRAPPER_DEFAULT_PATH)
+        confo.qemuwrapper_workdir = ConfDB().get('Qemu/qemuwrapper_working_directory', Defaults.QEMUWRAPPER_DEFAULT_WORKDIR)
         confo.qemu_path = ConfDB().get('Qemu/qemu_path', unicode('/usr/local/bin/qemu-system-i386'))
         confo.qemu_img_path = ConfDB().get('Qemu/qemu_img_path', unicode('/usr/local/bin/qemu-img'))
         confo.external_hosts = ConfDB().get('Qemu/external_hosts', unicode('localhost:10525')).split(',')
+        confo.enable_QemuWrapperAdvOptions = ConfDB().value("Qemu/enable_QemuWrapperAdvOptions", QVariant(False)).toBool()
         confo.enable_QemuManager = ConfDB().value("Qemu/enable_QemuManager", QVariant(True)).toBool()
         confo.import_use_QemuManager = ConfDB().value("Qemu/qemu_manager_import", QVariant(True)).toBool()
+        confo.send_path_external_QemuWrapper = ConfDB().value("Qemu/send_paths_external_Qemuwrapper", QVariant(False)).toBool()
         confo.QemuManager_binding = ConfDB().get('Qemu/qemu_manager_binding', unicode('127.0.0.1'))
         confo.qemuwrapper_port = int(ConfDB().get('Qemu/qemuwrapper_port', 10525))
         confo.qemuwrapper_baseUDP = int(ConfDB().get('Qemu/qemuwrapper_baseUDP', 20000))
         confo.qemuwrapper_baseConsole = int(ConfDB().get('Qemu/qemuwrapper_baseConsole', 3000))
 
-        # replace ~user and $HOME by home directory
+        # Qemu replace ~user and $HOME by home directory
         if os.environ.has_key("HOME"):
             confo.qemuwrapper_path = confo.qemuwrapper_path.replace('$HOME', os.environ["HOME"])
             confo.qemuwrapper_workdir =  confo.qemuwrapper_workdir.replace('$HOME', os.environ["HOME"])
 
         confo.qemuwrapper_path = os.path.expanduser(confo.qemuwrapper_path)
         confo.qemuwrapper_workdir = os.path.expanduser(confo.qemuwrapper_workdir)
-        
+
+        # VBox config
+
+        self.systconf['vbox'] = systemVBoxConf()
+        confo = self.systconf['vbox']
+        confo.vboxwrapper_path = ConfDB().get('VBox/vboxwrapper_path', Defaults.VBOXWRAPPER_DEFAULT_PATH)
+        confo.vboxwrapper_workdir = ConfDB().get('VBox/vboxwrapper_working_directory', Defaults.VBOXWRAPPER_DEFAULT_WORKDIR)
+        confo.external_hosts = ConfDB().get('VBox/external_hosts', unicode('localhost:11525')).split(',')
+        confo.enable_VBoxWrapperAdvOptions = ConfDB().value("VBox/enable_VBoxWrapperAdvOptions", QVariant(False)).toBool()
+        confo.enable_VBoxAdvOptions = ConfDB().value("VBox/enable_VBoxAdvOptions", QVariant(False)).toBool()
+        confo.enable_GuestControl = ConfDB().value("VBox/enable_GuestControl", QVariant(False)).toBool()
+        confo.enable_VBoxManager = ConfDB().value("VBox/enable_VBoxManager", QVariant(True)).toBool()
+        confo.import_use_VBoxManager = ConfDB().value("VBox/vbox_manager_import", QVariant(True)).toBool()
+        confo.VBoxManager_binding = ConfDB().get('VBox/vbox_manager_binding', unicode('127.0.0.1'))
+        confo.vboxwrapper_port = int(ConfDB().get('VBox/vboxwrapper_port', 11525))
+        confo.vboxwrapper_baseUDP = int(ConfDB().get('VBox/vboxwrapper_baseUDP', 20900))
+        confo.vboxwrapper_baseConsole = int(ConfDB().get('VBox/vboxwrapper_baseConsole', 3900))
+
+        # VBox replace ~user and $HOME by home directory
+        if os.environ.has_key("HOME"):
+            confo.vboxwrapper_path = confo.vboxwrapper_path.replace('$HOME', os.environ["HOME"])
+            confo.vboxwrapper_workdir =  confo.vboxwrapper_workdir.replace('$HOME', os.environ["HOME"])
+
+        confo.vboxwrapper_path = os.path.expanduser(confo.vboxwrapper_path)
+        confo.vboxwrapper_workdir = os.path.expanduser(confo.vboxwrapper_workdir)
+
         # Capture config
         self.systconf['capture'] = systemCaptureConf()
         confo = self.systconf['capture']
-        confo.workdir = ConfDB().get('Capture/working_directory', unicode(''))
-        confo.cap_cmd = ConfDB().get('Capture/capture_reader_cmd', unicode(''))
-        confo.auto_start = ConfDB().value('Capture/auto_start_cmd', QVariant(True)).toBool()
-        
+        confo.workdir = ConfDB().get('Capture/working_directory', Defaults.CAPTURE_DEFAULT_WORKDIR)
+        confo.cap_cmd = ConfDB().get('Capture/capture_reader_cmd', Defaults.CAPTURE_DEFAULT_CMD)
+        confo.auto_start = ConfDB().value('Capture/auto_start_cmd', QVariant(False)).toBool()
+
         # replace ~user and $HOME by home directory
         if os.environ.has_key("HOME"):
             confo.cap_cmd = confo.cap_cmd.replace('$HOME', os.environ["HOME"])
@@ -391,15 +491,23 @@ class Application(QApplication, Singleton):
         confo.project_startup = ConfDB().value("GNS3/project_startup", QVariant(True)).toBool()
         confo.relative_paths = ConfDB().value("GNS3/relative_paths", QVariant(True)).toBool()
         confo.use_shell = ConfDB().value("GNS3/use_shell", QVariant(True)).toBool()
-        confo.term_cmd = ConfDB().get('GNS3/console', unicode(''))
-        confo.project_path = ConfDB().get('GNS3/project_directory', unicode(''))
-        confo.ios_path = ConfDB().get('GNS3/ios_directory', unicode(''))
+        confo.bring_console_to_front = ConfDB().value("GNS3/bring_console_to_front", QVariant(False)).toBool()
+        confo.term_cmd = ConfDB().get('GNS3/console', Defaults.TERMINAL_DEFAULT_CMD)
+        confo.project_path = ConfDB().get('GNS3/project_directory', Defaults.PROJECT_DEFAULT_DIR)
+        confo.ios_path = ConfDB().get('GNS3/ios_directory', Defaults.IOS_DEFAULT_DIR)
         confo.status_points = ConfDB().value("GNS3/gui_show_status_points", QVariant(True)).toBool()
         confo.manual_connection =ConfDB().value("GNS3/gui_use_manual_connection", QVariant(False)).toBool()
         confo.draw_selected_rectangle = ConfDB().value("GNS3/gui_draw_selected_rectangle", QVariant(False)).toBool()
         confo.scene_width = int(ConfDB().get('GNS3/scene_width', 2000))
         confo.scene_height = int(ConfDB().get('GNS3/scene_height', 1000))
-        
+        confo.console_delay = float(ConfDB().get('GNS3/console_delay', 0.5))
+        if sys.platform.startswith('win') or sys.platform.startswith('darwin'):
+            # by default auto check for update only on Windows or OSX
+            confo.auto_check_for_update = ConfDB().value("GNS3/auto_check_for_update", QVariant(True)).toBool()
+        else:
+            confo.auto_check_for_update = ConfDB().value("GNS3/auto_check_for_update", QVariant(False)).toBool()
+        confo.last_check_for_update = int(ConfDB().get('GNS3/last_check_for_update', 0))
+
         # replace ~user and $HOME by home directory
         if os.environ.has_key("HOME"):
             confo.term_cmd = confo.term_cmd.replace('$HOME', os.environ["HOME"])
@@ -420,6 +528,10 @@ class Application(QApplication, Singleton):
         # QemuManager
         self.__QemuManager = QemuManager()
 
+        # VBoxManager
+        self.__VBoxManager = VBoxManager()
+
+        GNS_Conf().VBOX_images()
         GNS_Conf().IOS_images()
         GNS_Conf().IOS_hypervisors()
         GNS_Conf().QEMU_images()
@@ -429,7 +541,8 @@ class Application(QApplication, Singleton):
         GNS_Conf().IDS_images()
         GNS_Conf().Libraries()
         GNS_Conf().Symbols()
-        
+        GNS_Conf().RecentFiles()
+
         # Workspace create a ` Scene' object,
         # so it also set self.__topology
         self.__workspace = Workspace()
@@ -448,26 +561,35 @@ class Application(QApplication, Singleton):
         self.mainWindow.restoreState(ConfDB().value("GUIState/State").toByteArray())
         self.mainWindow.show()
 
-        configFile = unicode(ConfDB().fileName())
+        configFile = unicode(ConfDB().fileName(), 'utf-8', errors='replace')
+        # if we cannot find our config file, we start the Wizard dialog
         if not os.access(configFile, os.F_OK):
-            dialog = Wizard()
-            dialog.show()
-            dialog.raise_()
-            dialog.activateWindow()
-        
-        if file:
-            self.mainWindow.load_netfile(file)
-        elif confo.project_startup and os.access(configFile, os.F_OK):
-            dialog = ProjectDialog()
+            dialog = Wizard(parent=self.mainWindow)
             dialog.show()
             self.mainWindow.centerDialog(dialog)
             dialog.raise_()
             dialog.activateWindow()
+            self.mainWindow.raise_()
+            dialog.raise_()
+        else:
+            if file:
+                self.mainWindow.load_netfile(file)
+            elif confo.project_startup and os.access(configFile, os.F_OK):
+                dialog = ProjectDialog(parent=self.mainWindow, newProject=True)
+                dialog.show()
+                self.mainWindow.centerDialog(dialog)
+                dialog.raise_()
+                dialog.activateWindow()
+                self.mainWindow.raise_()
+                dialog.raise_()
+            else:
+                self.mainWindow.raise_()
 
         retcode = QApplication.exec_()
 
         self.__HypervisorManager = None
         self.__QemuManager = None
+        self.__VBoxManager = None
 
         if globals.recordConfiguration:
             # Save the geometry & state of the GUI
@@ -493,6 +615,7 @@ class Application(QApplication, Singleton):
         c.set('GNS3/autosave', confo.autosave)
         c.set('GNS3/console', confo.term_cmd)
         c.set('GNS3/use_shell', confo.use_shell)
+        c.set('GNS3/bring_console_to_front', confo.bring_console_to_front)
         c.set('GNS3/gui_show_status_points', confo.status_points)
         c.set('GNS3/gui_use_manual_connection', confo.manual_connection)
         c.set('GNS3/gui_draw_selected_rectangle', confo.draw_selected_rectangle)
@@ -500,6 +623,9 @@ class Application(QApplication, Singleton):
         c.set('GNS3/ios_directory', confo.ios_path)
         c.set('GNS3/scene_width', confo.scene_width)
         c.set('GNS3/scene_height', confo.scene_height)
+        c.set('GNS3/auto_check_for_update', confo.auto_check_for_update)
+        c.set('GNS3/last_check_for_update', confo.last_check_for_update)
+        c.set('GNS3/console_delay', confo.console_delay)
 
         # Dynamips settings
         confo = self.systconf['dynamips']
@@ -517,8 +643,9 @@ class Application(QApplication, Singleton):
         c.set('Dynamips/hypervisor_memory_usage_limit', confo.memory_limit)
         c.set('Dynamips/hypervisor_udp_incrementation', confo.udp_incrementation)
         c.set('Dynamips/hypervisor_manager_import', confo.import_use_HypervisorManager)
+        c.set('Dynamips/allocate_hypervisor_per_IOS', confo.allocateHypervisorPerIOS)
         c.set('Dynamips/hypervisor_manager_binding', confo.HypervisorManager_binding)
-        
+
         # Qemu config
         confo = self.systconf['qemu']
         c.set('Qemu/qemuwrapper_path', confo.qemuwrapper_path)
@@ -527,13 +654,31 @@ class Application(QApplication, Singleton):
         c.set('Qemu/qemu_img_path', confo.qemu_img_path)
         external_hosts = ','.join(confo.external_hosts)
         c.set('Qemu/external_hosts', external_hosts)
+        c.set('Qemu/enable_QemuWrapperAdvOptions', confo.enable_QemuWrapperAdvOptions)
         c.set('Qemu/enable_QemuManager', confo.enable_QemuManager)
         c.set('Qemu/qemu_manager_import', confo.import_use_QemuManager)
         c.set('Qemu/qemu_manager_binding', confo.QemuManager_binding)
+        c.set('Qemu/send_paths_external_Qemuwrapper', confo.send_path_external_QemuWrapper)
         c.set('Qemu/qemuwrapper_port', confo.qemuwrapper_port)
         c.set('Qemu/qemuwrapper_baseUDP', confo.qemuwrapper_baseUDP)
         c.set('Qemu/qemuwrapper_baseConsole', confo.qemuwrapper_baseConsole)
-        
+
+        # VBox config
+        confo = self.systconf['vbox']
+        c.set('VBox/vboxwrapper_path', confo.vboxwrapper_path)
+        c.set('VBox/vboxwrapper_working_directory', confo.vboxwrapper_workdir)
+        external_hosts = ','.join(confo.external_hosts)
+        c.set('VBox/external_hosts', external_hosts)
+        c.set('VBox/enable_VBoxWrapperAdvOptions', confo.enable_VBoxWrapperAdvOptions)
+        c.set('VBox/enable_VBoxAdvOptions', confo.enable_VBoxAdvOptions)
+        c.set('VBox/enable_GuestControl', confo.enable_GuestControl)
+        c.set('VBox/enable_VBoxManager', confo.enable_VBoxManager)
+        c.set('VBox/vbox_manager_import', confo.import_use_VBoxManager)
+        c.set('VBox/vbox_manager_binding', confo.VBoxManager_binding)
+        c.set('VBox/vboxwrapper_port', confo.vboxwrapper_port)
+        c.set('VBox/vboxwrapper_baseUDP', confo.vboxwrapper_baseUDP)
+        c.set('VBox/vboxwrapper_baseConsole', confo.vboxwrapper_baseConsole)
+
         # Capture settings
         confo = self.systconf['capture']
         c.set('Capture/working_directory', confo.workdir)
@@ -548,38 +693,47 @@ class Application(QApplication, Singleton):
         c.beginGroup("IOS.hypervisors")
         c.remove("")
         c.endGroup()
-        
+
         c.beginGroup("QEMU.images")
         c.remove("")
         c.endGroup()
-        
+
+        c.beginGroup("VBOX.images")
+        c.remove("")
+        c.endGroup()
+
         c.beginGroup("PIX.images")
         c.remove("")
         c.endGroup()
-        
+
         c.beginGroup("JUNOS.images")
         c.remove("")
         c.endGroup()
-        
+
         c.beginGroup("ASA.images")
         c.remove("")
         c.endGroup()
-        
+
         c.beginGroup("IDS.images")
         c.remove("")
         c.endGroup()
-        
+
         # Clear Symbol.libraries group
         c.beginGroup("Symbol.libraries")
         c.remove("")
         c.endGroup()
-        
+
+        # Clear Recent.files group
+        c.beginGroup("Recent.files")
+        c.remove("")
+        c.endGroup()
+
         # Clear Symbol.libraries group
         c.beginGroup("Symbol.settings")
         c.remove("")
         c.endGroup()
 
-        # IOS Images 
+        # IOS Images
         for (key, o) in self.__iosimages.iteritems():
             basekey = "IOS.images/" + str(o.id)
             c.set(basekey + "/filename", o.filename)
@@ -613,9 +767,18 @@ class Application(QApplication, Singleton):
             c.set(basekey + "/nic_nb", o.nic_nb)
             c.set(basekey + "/nic", o.nic)
             c.set(basekey + "/options", o.options)
-            c.set(basekey + "/kqemu", o.kqemu)
             c.set(basekey + "/kvm", o.kvm)
-            
+
+        # VBox images
+        for (key, o) in self.__vboximages.iteritems():
+            basekey = "VBOX.images/" + str(o.id)
+            c.set(basekey + "/name", o.name)
+            c.set(basekey + "/filename", o.filename)
+            c.set(basekey + "/nic_nb", o.nic_nb)
+            c.set(basekey + "/nic", o.nic)
+            c.set(basekey + "/guestcontrol_user", o.guestcontrol_user)
+            c.set(basekey + "/guestcontrol_password", o.guestcontrol_password)
+
         # PIX images
         for (key, o) in self.__piximages.iteritems():
             basekey = "PIX.images/" + str(o.id)
@@ -625,10 +788,9 @@ class Application(QApplication, Singleton):
             c.set(basekey + "/nic_nb", o.nic_nb)
             c.set(basekey + "/nic", o.nic)
             c.set(basekey + "/options", o.options)
-            c.set(basekey + "/kqemu", o.kqemu)
             c.set(basekey + "/key", o.key)
             c.set(basekey + "/serial", o.serial)
-            
+
         # JunOS images
         for (key, o) in self.__junosimages.iteritems():
             basekey = "JUNOS.images/" + str(o.id)
@@ -638,9 +800,8 @@ class Application(QApplication, Singleton):
             c.set(basekey + "/nic_nb", o.nic_nb)
             c.set(basekey + "/nic", o.nic)
             c.set(basekey + "/options", o.options)
-            c.set(basekey + "/kqemu", o.kqemu)
             c.set(basekey + "/kvm", o.kvm)
-            
+
         # ASA images
         for (key, o) in self.__asaimages.iteritems():
             basekey = "ASA.images/" + str(o.id)
@@ -649,12 +810,11 @@ class Application(QApplication, Singleton):
             c.set(basekey + "/nic_nb", o.nic_nb)
             c.set(basekey + "/nic", o.nic)
             c.set(basekey + "/options", o.options)
-            c.set(basekey + "/kqemu", o.kqemu)
             c.set(basekey + "/kvm", o.kvm)
             c.set(basekey + "/initrd", o.initrd)
             c.set(basekey + "/kernel", o.kernel)
             c.set(basekey + "/kernel_cmdline", o.kernel_cmdline)
-            
+
         # IDS images
         for (key, o) in self.__idsimages.iteritems():
             basekey = "IDS.images/" + str(o.id)
@@ -665,7 +825,6 @@ class Application(QApplication, Singleton):
             c.set(basekey + "/nic_nb", o.nic_nb)
             c.set(basekey + "/nic", o.nic)
             c.set(basekey + "/options", o.options)
-            c.set(basekey + "/kqemu", o.kqemu)
             c.set(basekey + "/kvm", o.kvm)
 
         # Libraries
@@ -674,7 +833,14 @@ class Application(QApplication, Singleton):
             basekey = "Symbol.libraries/" + str(id)
             c.set(basekey + "/path", o.path)
             id += 1
-            
+
+        # Recent Files
+        id = 0
+        for o in self.__recentfiles:
+            basekey = "Recent.files/" + str(id)
+            c.set(basekey + "/path", o.path)
+            id += 1
+
         # Symbols
         id = 0
         for symbol in SYMBOLS:
