@@ -1,5 +1,6 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+# vim: expandtab ts=4 sw=4 sts=4:
 
 """
 confConsole.py
@@ -28,6 +29,12 @@ from dynamips_lib import DynamipsError, DynamipsWarning, GENERIC_7200_PAS, GENER
 # Regex matching slot definitions. Used in pre_cmd() function in confRouterConsole
 SLOT_RE = re.compile('^slot[0-7]', re.IGNORECASE)
 
+# True = Dynagen text-mode, False = GNS3 GUI-mode.
+if __name__ == 'confConsole':
+    PureDynagen = True
+else:
+    PureDynagen = False
+
 # determine if we are in the debugger
 try:
     DBGPHideChildren
@@ -120,18 +127,19 @@ class AbstractConsole(cmd.Cmd):
 
         pass
 
-    def do_py(self, line):
-        """py <python statement(s)>
+    if PureDynagen:
+        def do_py(self, line):
+            """py <python statement(s)>
 \tExecute python statements"""
 
-        if line == '?':
-            print self.do_py.__doc__
-            return
+            if line == '?':
+                print self.do_py.__doc__
+                return
 
-        try:
-            exec line in self._locals, self._globals
-        except Exception, e:
-            print e.__class__, ':', e
+            try:
+                exec line in self._locals, self._globals
+            except Exception, e:
+                print e.__class__, ':', e
 
     def default(self, line):
         """Called on an input line when the command prefix is not recognized.
@@ -145,10 +153,11 @@ class AbstractConsole(cmd.Cmd):
 
         print self._hist
 
-    def do_exit(self, args):
-        """Exits from the console"""
+    if PureDynagen:
+        def do_exit(self, args):
+            """Exits from the console"""
 
-        return -1
+            return -1
 
     def do_end(self, args):
         """Exits from the console"""
@@ -1045,7 +1054,7 @@ no <option> = <option_value>
                 error(e)
         else:
             error('the only possible options are True or False, not: ' + args)
-            
+
     def do_slot(self, args):
         if '?' in args or args.strip() == '':
             print self.do_slot.__doc__
@@ -1384,7 +1393,7 @@ class confATMBRConsole(AbstractConfSWConsole):
             if len(right_side) == 3:
                 try:
                     port1 = int(left_side)
-                    port2 = int(right_side[0])                  
+                    port2 = int(right_side[0])
                     vpi2 = int(right_side[1])
                     #1 = 2:0:201 R1 a1/0
                     try:
