@@ -1,9 +1,10 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+# vim: expandtab ts=4 sw=4 sts=4:
 
 """
 console.py
-Copyright (C) 2006-2010  Greg Anuzelli
+Copyright (C) 2006-2011  Greg Anuzelli
 contributions: Pavel Skovajsa
 
 Derived from recipe on ASPN Cookbook
@@ -34,6 +35,12 @@ from dynamips_lib import Router, Emulated_switch, DynamipsError, DynamipsWarning
 from configobj import ConfigObj
 from confConsole import AbstractConsole, confHypervisorConsole, confConsole
 
+# True = Dynagen text-mode, False = GNS3 GUI-mode.
+if __name__ == 'console':
+    PureDynagen = True
+else:
+    PureDynagen = False
+
 # determine if we are in the debugger
 try:
     DBGPHideChildren
@@ -56,7 +63,7 @@ class progressBar:
         self.max = maxValue
         self.span = maxValue - minValue
         self.width = totalWidth
-        self.amount = 0       # When amount == max, we are 100% done 
+        self.amount = 0       # When amount == max, we are 100% done
         self.updateAmount(0)  # Build progress bar string
 
     def updateAmount(self, newAmount = 0):
@@ -79,7 +86,7 @@ class progressBar:
         self.progBar = "[" + '#'*numHashes + ' '*(allFull-numHashes) + "]"
 
         # figure out where to put the percentage, roughly centered
-        percentPlace = (len(self.progBar) / 2) - len(str(percentDone)) 
+        percentPlace = (len(self.progBar) / 2) - len(str(percentDone))
         percentString = str(percentDone) + "%"
 
         # slice the percentage into the bar
@@ -95,7 +102,7 @@ class Console(AbstractConsole):
     def __init__(self, dynagen):
         AbstractConsole.__init__(self)
         self.prompt = '=> '
-        self.intro  = 'Dynagen management console for Dynamips and Qemuwrapper ' + self.namespace.VERSION + '\nCopyright (c) 2005-2010 Greg Anuzelli, contributions Pavel Skovajsa & Jeremy Grossman\n'
+        self.intro  = 'Dynagen management console for Dynamips and Qemuwrapper/VBoxwrapper ' + self.namespace.VERSION + '\nCopyright (c) 2005-2011 Greg Anuzelli, contributions Pavel Skovajsa, Jeremy Grossmann & Alexey Eromenko "Technologov"\n'
         self.dynagen = dynagen
     ## Command definitions ##
 
@@ -119,12 +126,13 @@ class Console(AbstractConsole):
 \tList all devices"""
 
         table = []
-        print '%-10s %-10s %-10s %-15s %-10s' % (
+        print '%-10s %-10s %-10s %-27s %-10s %-10s' % (
             'Name',
             'Type',
             'State',
             'Server',
             'Console',
+            'AUX'
             )
         for device in self.dynagen.devices.values():
             row = []
@@ -140,13 +148,18 @@ class Console(AbstractConsole):
                 row.append('%-10s' % 'always on')
             try:
                 server = device.dynamips.host + ":" + str(device.dynamips.port)
-                row.append('%-15s' % server)
+                row.append('%-20s' % server)
             except AttributeError:
-                row.append('%-15s' % 'n/a')
+                row.append('%-20s' % 'n/a')
             try:
                 row.append('%-10s' % device.console)
             except AttributeError:
                 row.append('%-10s' % 'n/a')
+            try:
+                row.append('%-10s' % device.aux)
+            except AttributeError:
+                row.append('%-10s' % 'n/a')
+
             table.append(row)
         table.sort(con_cmp)  # Sort the table by the console port #
         for line in table:
@@ -154,57 +167,58 @@ class Console(AbstractConsole):
                 print item,
             print
 
-    def do_conf(self, args):
-        """conf <hypervisor address>:<hypervisor port>
-\tswitch into configuration mode of the specific hypervisor eg. 'conf localhost'. If the hypervisor does not exist it will be created.
-conf
-\tswitch into global config mode"""
+    if PureDynagen:
+        def do_conf(self, args):
+            """conf <hypervisor address>:<hypervisor port>
+    \tswitch into configuration mode of the specific hypervisor eg. 'conf localhost'. If the hypervisor does not exist it will be created.
+    conf
+    \tswitch into global config mode"""
 
-        if '?' in args:
-            print self.do_conf.__doc__
-            return
-
-        #if this is a conf <nothing> command go into global config mode
-        if args.strip() == "":
-            nested_cmd = confConsole(self.dynagen, self)
-            nested_cmd.cmdloop()
-            return
-
-        #if this is a conf <hypervisor_name> go into hypervisor config mode
-        #check if this hypervisor already exists
-        found = False
-        params = args.split(":")
-        if len(params) == 1:
-            hyp_name = params[0]
-            hyp_port = 7200
-        elif len(params) == 2:
-            try:
-                hyp_name = params[0]
-                hyp_port = int(params[1])
-            except (AttributeError, ValueError):
-                error('Syntax error in ' + args + ' . Use <hypervisor address>:<hypervisor port> syntax')
+            if '?' in args:
+                print self.do_conf.__doc__
                 return
-        else:
-            error('Syntax error in ' + params + ' . Use <hypervisor address>:<hypervisor port> syntax')
-            return
 
-        for server in self.dynagen.dynamips.values():
-            if hyp_name == server.host and hyp_port == server.port:
-                found = True
-                break
-        if not found:
-            #if not found create the hypervisor instance...
-            dynamips = self.dynagen.create_dynamips_hypervisor(hyp_name, hyp_port)
-
-            #call hypervisor config mode
-            if dynamips != None:
-                nested_cmd = confHypervisorConsole(dynamips, self.dynagen)
+            #if this is a conf <nothing> command go into global config mode
+            if args.strip() == "":
+                nested_cmd = confConsole(self.dynagen, self)
                 nested_cmd.cmdloop()
-        else:
+                return
 
-            #looks like we found an already existing hypervisor instance, so let's jump into nested conf Cmd to configure it
-            nested_cmd = confHypervisorConsole(server, self.dynagen)
-            nested_cmd.cmdloop()
+            #if this is a conf <hypervisor_name> go into hypervisor config mode
+            #check if this hypervisor already exists
+            found = False
+            params = args.split(":")
+            if len(params) == 1:
+                hyp_name = params[0]
+                hyp_port = 7200
+            elif len(params) == 2:
+                try:
+                    hyp_name = params[0]
+                    hyp_port = int(params[1])
+                except (AttributeError, ValueError):
+                    error('Syntax error in ' + args + ' . Use <hypervisor address>:<hypervisor port> syntax')
+                    return
+            else:
+                error('Syntax error in ' + params + ' . Use <hypervisor address>:<hypervisor port> syntax')
+                return
+
+            for server in self.dynagen.dynamips.values():
+                if hyp_name == server.host and hyp_port == server.port:
+                    found = True
+                    break
+            if not found:
+                #if not found create the hypervisor instance...
+                dynamips = self.dynagen.create_dynamips_hypervisor(hyp_name, hyp_port)
+
+                #call hypervisor config mode
+                if dynamips != None:
+                    nested_cmd = confHypervisorConsole(dynamips, self.dynagen)
+                    nested_cmd.cmdloop()
+            else:
+
+                #looks like we found an already existing hypervisor instance, so let's jump into nested conf Cmd to configure it
+                nested_cmd = confHypervisorConsole(server, self.dynagen)
+                nested_cmd.cmdloop()
 
     def do_suspend(self, args):
         """suspend  {/all | router1 [router2] ...}
@@ -244,6 +258,44 @@ conf
                 print "Note: " + str(e)
 
 
+    def do_vboxexec(self, args):
+        """vboxexec <VBOX device> <command>\nVirtualBox GuestControl execute sends a command to VirtualBox guest and prints it's output (experimental feature).
+This requires VirtualBox Guest Additions to be installed inside the guest VM."""
+
+        if '?' in args or args.strip() == "":
+            print self.do_vboxexec.__doc__
+            return
+        devices = args.split(" ")
+
+        #if devname in devices[0]:
+        devname =  devices[0]
+        #print "ADEBUG: console.py: devname = ", devname
+
+        try:
+            device = self.dynagen.devices[devname]
+            if not isinstance(device, self.namespace.AnyVBoxEmuDevice):
+                error("Device is not VirtualBox device: " + devname)
+                return
+            result = device.vboxexec(args.split(" ")[1:])
+            #print "ADEBUG: console.py: vboxexec raw result = ", result
+            #If we got incorrect result, just drop it.
+            try:
+                if not result[0][0:10] == "100-result":
+                    #print "ADEBUG: console.py: result[0][0:9] = %s" % result[0][0:9]
+                    return
+            except:
+                return
+            # vboxwrapper TCP server incorrectly formats text by adding double
+            #line-ending in UNIX style, which we convert to single line-end in client-native style
+            print result[0][10:].replace('\n\n', os.linesep)
+        except IndexError:
+            pass
+        except (KeyError, AttributeError):
+            error('invalid device: ' + devname)
+        except DynamipsError, e:
+            error(e)
+        except DynamipsWarning, e:
+            print "Note: " + str(e)
 
     def do_start(self, args):
         """start  {/all [delay] | router1 [router2] ...}
@@ -252,7 +304,7 @@ conf
         """
 
         startdelay = self.dynagen.startdelay
-        
+
         if '?' in args or args.strip() == "":
             print self.do_start.__doc__
             return
@@ -443,19 +495,21 @@ conf
             print ' %s at %s:%i has version %s' % (d.type, d.host, d.port, d.version)
         print """
 Credits:
-Dynagen is written and maintained by Greg Anuzelli
-Contributing developer: Pavel Skovajsa & Jeremy Grossmann
+Dynagen is written by Greg Anuzelli
+Contributing developers: Pavel Skovajsa, Jeremy Grossmann & Alexey Eromenko "Technologov"
 Qemuwrapper: Thomas Pani & Jeremy Grossmann
+VBoxwrapper: Thomas Pani, Jeremy Grossmann & Alexey Eromenko "Technologov"
 Pemu: Milen Svobodnikov
 Thanks to the authors of the ConfObj library
 
 And big thanks of course to Christophe Fillot as the author of Dynamips.
 """
 
-    def do_shell(self, args):
-        """Pass command to a system shell when line begins with '!'"""
+    if PureDynagen:
+        def do_shell(self, args):
+            """Pass command to a system shell when line begins with '!'"""
 
-        os.system(args)
+            os.system(args)
 
     def do_telnet(self, args):
         """telnet  {/all | router1 [router2] ...}
@@ -507,6 +561,46 @@ And big thanks of course to Christophe Fillot as the author of Dynamips.
             except DynamipsWarning, e:
                 print "Note: " + str(e)
 
+    def do_aux(self, args):
+        """aux  {/all | router1 [router2] ...}
+\tconnect to the AUX port(s) of all or a specific router(s)
+        """
+
+        if '?' in args or args.strip() == "":
+            print self.do_aux.__doc__
+            return
+
+        devices = args.split(" ")
+        if '/all' in args.split(" "):
+            # Set devices to all the devices
+            devices = self.dynagen.devices.values()
+        else:
+            devices = []
+            for device in args.split(" "):
+                # Create a list of all the device objects
+                try:
+                    devices.append(self.dynagen.devices[device])
+                except KeyError:
+                    error('unknown device: ' + device)
+
+        for device in devices:
+            try:
+                if not device.isrouter:
+                    continue
+
+                if device.state != 'running':
+                    print 'Skipping %s device: %s' % (device.state, device.name)
+                    continue
+                self.aux(device.name)
+            except IndexError:
+                pass
+            except (KeyError, AttributeError):
+                error('invalid device: ' + device.name)
+            except DynamipsError, e:
+                error(e)
+            except DynamipsWarning, e:
+                print "Note: " + str(e)
+
     def show_device(self, params):
         """show device {something} command prints the output by calling get_device_info function"""
 
@@ -514,7 +608,7 @@ And big thanks of course to Christophe Fillot as the author of Dynamips.
             output = []
             for device in self.dynagen.devices.values():
                 #if it is a router or other emulated device
-                if isinstance(device, (self.namespace.Router, self.namespace.AnyEmuDevice, self.namespace.FRSW, self.namespace.ATMBR, self.namespace.ATMSW, self.namespace.ETHSW)):
+                if isinstance(device, (self.namespace.Router, self.namespace.AnyEmuDevice, self.namespace.AnyVBoxEmuDevice, self.namespace.FRSW, self.namespace.ATMBR, self.namespace.ATMSW, self.namespace.ETHSW)):
                     output.append(device.info())
             output.sort()
             for devinfo in output:
@@ -523,7 +617,7 @@ And big thanks of course to Christophe Fillot as the author of Dynamips.
             #if this is 'show device {something}' command print info about specific device
             try:
                 device = self.dynagen.devices[params[1]]
-                if isinstance(device, (self.namespace.Router, self.namespace.AnyEmuDevice, self.namespace.FRSW, self.namespace.ATMBR, self.namespace.ATMSW, self.namespace.ETHSW)):
+                if isinstance(device, (self.namespace.Router, self.namespace.AnyEmuDevice, self.namespace.AnyVBoxEmuDevice, self.namespace.FRSW, self.namespace.ATMBR, self.namespace.ATMSW, self.namespace.ETHSW)):
                     print device.info()
             except KeyError:
                 error('unknown device: ' + params[1])
@@ -783,9 +877,9 @@ show run <device_name>
 
         # Set the current directory to the one that contains our network file
         try:
-            netdir = os.getcwd()
+            netdir = os.getcwdu()
             subdir = os.path.dirname(self.dynagen.global_filename)
-            self.debug('current dir is -> ' + os.getcwd())
+            self.debug('current dir is -> ' + os.getcwdu())
             if subdir != "":
                 self.debug("changing dir to -> " + subdir)
                 os.chdir(subdir)
@@ -850,9 +944,9 @@ show run <device_name>
 
         # Set the current directory to the one that contains our network file
         try:
-            netdir = os.getcwd()
+            netdir = os.getcwdu()
             subdir = os.path.dirname(self.dynagen.global_filename)
-            self.debug('current dir is -> ' + os.getcwd())
+            self.debug('current dir is -> ' + os.getcwdu())
             if subdir != "":
                 self.debug("changing dir to -> " + subdir)
                 os.chdir(subdir)
@@ -1021,11 +1115,13 @@ show run <device_name>
         except ValueError:
             print 'Error parsing command'
             return
+
     def do_send(self, args):
         """send [host] commandstring
 \tsend a raw hypervisor command to a dynamips server
+\tplease use 'list' to find available hosts
 \tExamples:
-\tsend bender hypervisor version   -- Send the 'hypervisor version' command to the host named bender"""
+\tsend 127.0.0.1 hypervisor version   -- Send the 'hypervisor version' command to the host named bender"""
 
         if '?' in args or args.strip() == "":
             print self.do_send.__doc__
@@ -1331,63 +1427,79 @@ Examples:
             except DynamipsWarning, e:
                 print "Note: " + str(e)
 
+    if PureDynagen:
+        def do_cpuinfo(self, args):
+            """cpuinfo  {/all | router1 [router2] ...}\nshow CPU info for a specific router(s)"""
 
-    def do_cpuinfo(self, args):
-        """cpuinfo  {/all | router1 [router2] ...}\nshow CPU info for a specific router(s)"""
+            if '?' in args or args.strip() == '':
+                print self.do_cpuinfo.__doc__
+                return
 
-        if '?' in args or args.strip() == '':
-            print self.do_cpuinfo.__doc__
-            return
+            devices = args.split(' ')
+            if '/all' in devices:
+                for device in self.dynagen.devices.values():
+                    try:
+                        for line in device.cpuinfo:
+                            print line.strip()
+                    except IndexError:
+                        pass
+                    except AttributeError:
+                        # If this device doesn't support cpuinfo just ignore it
+                        pass
+                    except DynamipsError, e:
+                        error(e)
+                    except DynamipsWarning, e:
+                        print "Note: " + str(e)
+                        return
+                return
 
-        devices = args.split(' ')
-        if '/all' in devices:
-            for device in self.dynagen.devices.values():
+            for device in devices:
                 try:
-                    for line in device.cpuinfo:
-                        print line.strip()
+                    print self.dynagen.devices[device].cpuinfo[0].strip()
                 except IndexError:
                     pass
-                except AttributeError:
-                    # If this device doesn't support cpuinfo just ignore it
-                    pass
+                except (KeyError, AttributeError):
+                    error('invalid device: ' + device)
                 except DynamipsError, e:
                     error(e)
                 except DynamipsWarning, e:
                     print "Note: " + str(e)
-                    return
-            return
-
-        for device in devices:
-            try:
-                print self.dynagen.devices[device].cpuinfo[0].strip()
-            except IndexError:
-                pass
-            except (KeyError, AttributeError):
-                error('invalid device: ' + device)
-            except DynamipsError, e:
-                error(e)
-            except DynamipsWarning, e:
-                print "Note: " + str(e)
 
     def telnet(self, device):
         """Telnet to the console port of device"""
-    
+
         import dynagen as dyn
         telnetstring = dyn.telnetstring
         port = str(self.dynagen.devices[device].console)
         host = str(self.dynagen.devices[device].dynamips.host)
-    
+
         if telnetstring and not dyn.notelnet:
             telnetstring = telnetstring.replace('%h', host)
             telnetstring = telnetstring.replace('%p', port)
             telnetstring = telnetstring.replace('%d', device)
-    
+
+            os.system(telnetstring)
+            time.sleep(0.5)  # Give the telnet client a chance to start
+
+    def aux(self, device):
+        """Telnet to the AUX port of device"""
+
+        import dynagen as dyn
+        telnetstring = dyn.telnetstring
+        port = str(self.dynagen.devices[device].aux)
+        host = str(self.dynagen.devices[device].dynamips.host)
+
+        if telnetstring and not dyn.notelnet:
+            telnetstring = telnetstring.replace('%h', host)
+            telnetstring = telnetstring.replace('%p', port)
+            telnetstring = telnetstring.replace('%d', device)
+
             os.system(telnetstring)
             time.sleep(0.5)  # Give the telnet client a chance to start
 
     def debug(self, string):
         """ Print string if debugging is true"""
-    
+
         # Debug level 2, console debugs
         if self.dynagen.debuglevel >= 2:
             print '  DEBUG: ' + str(string)
