@@ -16,53 +16,67 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
+#debuglevel: 0=disabled, 1=default, 2=debug, 3=deep debug
+debuglevel = 0
+
 import sys, os, re
 import GNS3.Globals as globals
 from GNS3.Dynagen.dynagen import Dynagen, DEVICETUPLE
 from GNS3.Utils import translate, debug, getWindowsInterfaces
 from PyQt4 import QtCore, QtGui
 
+def debugmsg(level, message):
+    if debuglevel == 0:
+        return
+    if debuglevel >= level:
+        print message
+
 class DynagenSub(Dynagen):
     """ Subclass of Dynagen
     """
 
     def __init__(self):
-
+        debugmsg(2, "DynagenSub::__init__()")
         Dynagen.__init__(self)
         self.gns3_data = None
+        self.rpcap_mapping = {}
 
     def check_replace_GUID_NIO(self, filename):
         """ Check and replace non-existing GUID (network interface ID) on Windows
         """
-        
+        debugmsg(2, "DynagenSub::check_replace_GUID_NIO(%s)" % unicode(filename))
+
         file = open(filename,'r')
         lines = file.readlines()
         cregex = re.compile("^.*nio_gen_eth:(.*)")
         niolist = []
-        
+
         for currentline in lines:
-            currentline = currentline.lower().strip() 
+            currentline = currentline.lower().strip()
             match_obj = cregex.match(currentline)
             if match_obj:
                 niolist.append(match_obj.group(1))
-                
+
+        self.rpcap_mapping.clear()
         niolist = set(niolist)
         if len(niolist):
-            
+
             rpcaps = getWindowsInterfaces()
             interfaces = {}
             for rpcap in rpcaps:
-                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*}).*:(.*)""", rpcap)
+                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*})\ :\ (.*)""", rpcap)
                 interface_guid = str(match.group(1)).lower()
                 interfaces[interface_guid] = unicode(match.group(2)).strip()
+                name_match = re.search(r"""^(.*)\ on local host:.*""", match.group(2))
+                self.rpcap_mapping['nio_gen_eth:' + interface_guid] = name_match.group(1)
 
             for nio in niolist:
-                if not interfaces.has_key(nio): 
-                    (selection, ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("DynagenSub", "NIO connection"),
-                                                        unicode(translate("DynagenSub", "%s cannot be found\nPlease choose an alternate network interface:")) % nio, interfaces.values(), 0, False)
+                if not interfaces.has_key(nio):
+                    (selection, ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("DynagenSub", "NIC binding"),
+                                                                 translate("DynagenSub", "NIC ID %s cannot be found on this host\nPlease choose an alternate network interface:") % nio, interfaces.values(), 0, False)
                     if ok:
                         interface = ""
                         for (key, name) in interfaces.iteritems():
@@ -76,30 +90,42 @@ class DynagenSub(Dynagen):
                             if match_obj:
                                 currentline = currentline.replace(nio, interface)
                                 lines[index] = currentline
-                            index += 1      
+                            index += 1
                     else:
                         continue
-  
+
         file.close()
-        
+
         if len(niolist):
             # write changes
             file = open(filename,'w')
             for line in lines:
                 file.write(line)
             file.close()
-          
+
+    def getHost(self, i_strAddress):
+        # IPv6: gets the "host" portion from "host:port" string
+        elements = i_strAddress.split(':')
+        for x in range(len(elements)-1): #Except TCP port
+            if x == 0:
+                hostname = elements[x]
+            else:
+                hostname += ':' + elements[x]
+        return hostname
 
     def open_config(self, FILENAME):
         """ Open the config file
         """
+        debugmsg(2, "DynagenSub::open_config(%s)" % unicode(FILENAME))
 
         if sys.platform.startswith('win'):
             self.check_replace_GUID_NIO(FILENAME)
         config = Dynagen.open_config(self, FILENAME)
+        debugmsg(2, ("DynagenSub::open_config(): config = ", config))
         self.filename = FILENAME
         self.gns3_data = None
         if 'GNS3-DATA' in config.sections:
+            debugmsg(3, "DynagenSub::open_config(), entered into: if 'GNS3-DATA'")
             self.gns3_data = config['GNS3-DATA'].copy()
             if self.gns3_data.has_key('configs'):
                 if os.path.exists(self.gns3_data['configs']):
@@ -121,6 +147,7 @@ class DynagenSub(Dynagen):
         progress.setWindowModality(QtCore.Qt.WindowModal)
         globals.GApp.processEvents(QtCore.QEventLoop.AllEvents)
         current = 0
+        debugmsg(3, ("DynagenSub::open_config(), config.sections = ", config.sections))
         for section in config.sections:
             progress.setValue(current)
             globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
@@ -129,14 +156,19 @@ class DynagenSub(Dynagen):
                 break
 
             server = config[section]
+            #debugmsg(3, ("DynagenSub::open_config(), server = ", server))  # Returns long config of hypervisor
             if ' ' in server.name:
-                (emulator, host) = server.name.split(' ')              
+                (emulator, host) = server.name.split(' ')
+                debugmsg(2, "DynagenSub::open_config(), emulator = %s, host = %s" % (unicode(emulator), unicode(host)))
                 if ':' in host:
                     # unpack the server and port
                     # controlPort is ignored
-                    (host, controlPort) = host.split(':')
+                    #(host, controlPort) = host.split(':')  # What if we provide IPv6 ???
+                    controlPort = host.split(':')[-1]
+                    host = self.getHost(host)
                 if emulator == 'qemu' and (host == globals.GApp.systconf['qemu'].QemuManager_binding or host == 'localhost') and globals.GApp.systconf['qemu'].enable_QemuManager:
                     globals.GApp.QemuManager.startQemu(int(controlPort))
+                    debugmsg(2, "DynagenSub::open_config(), entered QemuManager")
 
                     # Check if this is a relative working directory path and convert to an absolute path if necessary
                     if server['workingdir']:
@@ -144,7 +176,7 @@ class DynagenSub(Dynagen):
                         if os.path.exists(abspath):
                             server['workingdir'] = abspath
                             debug(unicode("Converting relative working directory path to absolute path: %s") % server['workingdir'])
-                    
+
                     if server['workingdir'] == '.':
                         server['workingdir'] = os.path.dirname(FILENAME)
 
@@ -152,55 +184,57 @@ class DynagenSub(Dynagen):
                         device = server[subsection]
                         # ASA has no image
                         if device.name == '5520' and device['initrd'] and device['kernel']:
+                            debugmsg(2, "DynagenSub::open_config(), entered QemuManager, ASA 5520")
                             if not os.access(device['initrd'], os.F_OK):
-                                
+
                                 if len(globals.GApp.asaimages.keys()):
                                     initrd_name = globals.GApp.asaimages.values()[0].initrd
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                        unicode(translate("ASA initrd", "ASA initrd %s cannot be found and cannot find an alternative initrd")) % device['initrd'])
+                                                               translate("ASA initrd", "ASA initrd %s cannot be found and cannot find an alternative initrd") % device['initrd'])
                                     continue
-                                print unicode(translate("DynagenSub", "Local ASA initrd %s cannot be found, use initrd %s instead")) \
-                                % (unicode(device['initrd']), initrd_name)
+                                print translate("DynagenSub", "Local ASA initrd %s cannot be found, use initrd %s instead") \
+                                                % (unicode(device['initrd']), initrd_name)
                                 device['initrd'] = initrd_name
-                                
+
                             if not os.access(device['kernel'], os.F_OK):
                                 if len(globals.GApp.asaimages.keys()):
                                     kernel_name = globals.GApp.asaimages.values()[0].kernel
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                        unicode(translate("ASA kernel", "ASA kernel %s cannot be found and cannot find an alternative kernel")) % device['kernel'])
+                                                               translate("ASA kernel", "ASA kernel %s cannot be found and cannot find an alternative kernel") % device['kernel'])
                                     continue
-                                print unicode(translate("DynagenSub", "Local ASA kernel %s cannot be found, use kernel %s instead")) \
-                                % (unicode(device['kernel']), kernel_name)
+                                print translate("DynagenSub", "Local ASA kernel %s cannot be found, use kernel %s instead") \
+                                                % (unicode(device['kernel']), kernel_name)
                                 device['kernel'] = kernel_name
                             continue
-                        
+
                         # IDS has no default image
                         if device.name == 'IDS-4215' and device['image1'] and device['image2']:
+                            debugmsg(2, "DynagenSub::open_config(), entered QemuManager, IDS")
                             if not os.access(device['image1'], os.F_OK):
-                                
+
                                 if len(globals.GApp.idsimages.keys()):
                                     image1_name = globals.GApp.idsimages.values()[0].image1
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                        unicode(translate("IDS image (hda)", "IDS image %s cannot be found and cannot find an alternative image")) % device['image1'])
+                                                               translate("IDS image (hda)", "IDS image %s cannot be found and cannot find an alternative image") % device['image1'])
                                     continue
-                                print unicode(translate("DynagenSub", "Local IDS image %s cannot be found, use image %s instead")) \
+                                print translate("DynagenSub", "Local IDS image %s cannot be found, use image %s instead") \
                                 % (unicode(device['image1']), image1_name)
                                 device['image1'] = image1_name
-                                
+
                             if not os.access(device['image2'], os.F_OK):
                                 if len(globals.GApp.idsimages.keys()):
                                     image2_name = globals.GApp.idsimages.values()[0].image2
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                        unicode(translate("IDS image (hdb)", "IDS image %s cannot be found and cannot find an alternative image")) % device['image2'])
+                                                               translate("IDS image (hdb)", "IDS image %s cannot be found and cannot find an alternative image") % device['image2'])
                                     continue
-                                print unicode(translate("DynagenSub", "Local IDS image %s cannot be found, use image %s instead")) \
+                                print translate("DynagenSub", "Local IDS image %s cannot be found, use image %s instead") \
                                 % (unicode(device['image2']), image2_name)
                                 device['image2'] = image2_name
-                                
+
                             continue
 
                         if device.name not in ('525', 'O-series', 'QemuDevice'):
@@ -210,31 +244,34 @@ class DynagenSub(Dynagen):
                             abspath = os.path.join(os.path.dirname(FILENAME), unicode(device['image']))
                             if os.path.exists(abspath):
                                 device['image'] = abspath
-                        
+
                         if device.name == 'O-series' and device['image']:
+                            debugmsg(2, "DynagenSub::open_config(), entered QemuManager, JunOS")
                             if not os.access(device['image'], os.F_OK):
                                 if len(globals.GApp.junosimages.keys()):
                                     image_name = globals.GApp.junosimages.values()[0].filename
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                        unicode(translate("JunOS image", "JunOS image %s cannot be found and cannot find an alternative image")) % device['image'])
+                                                               translate("JunOS image", "JunOS image %s cannot be found and cannot find an alternative image") % device['image'])
                                     continue
-                                print unicode(translate("DynagenSub", "Local JunOS image %s cannot be found, use image %s instead")) \
-                                % (unicode(device['image']), image_name)
+                                print translate("DynagenSub", "Local JunOS image %s cannot be found, use image %s instead") \
+                                                % (unicode(device['image']), image_name)
                                 device['image'] = image_name
 
                         if device.name == 'QemuDevice' and device['image']:
+                            debugmsg(2, "DynagenSub::open_config(), entered QemuManager, QemuDevice")
                             if not os.access(device['image'], os.F_OK):
                                 if len(globals.GApp.qemuimages.keys()):
                                     image_name = globals.GApp.qemuimages.values()[0].filename
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                        unicode(translate("Qemu image", "Qemu host image %s cannot be found and cannot find an alternative image")) % device['image'])
-                                    continue                                    
-                                print unicode(translate("DynagenSub", "Local Qemu host image %s cannot be found, use image %s instead")) \
+                                                               translate("Qemu image", "Qemu guest image %s cannot be found and cannot find an alternative image") % device['image'])
+                                    continue
+                                print translate("DynagenSub", "Local Qemu guest image %s cannot be found, use image %s instead") \
                                 % (unicode(device['image']), image_name)
                                 device['image'] = image_name
                         elif device['image']:
+                            debugmsg(2, "DynagenSub::open_config(), entered QemuManager, elif device, possibly PIX")
                             # must be a PIX device
                             # check if the PIX image is accessible, if not find an alternative image
                             if not os.access(device['image'], os.F_OK):
@@ -242,17 +279,55 @@ class DynagenSub(Dynagen):
                                     image_name = globals.GApp.piximages.values()[0].filename
                                 else:
                                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                        unicode(translate("PIX image", "PIX image %s cannot be found and cannot find an alternative image")) % device['image'])
+                                                               translate("PIX image", "PIX image %s cannot be found and cannot find an alternative image") % device['image'])
                                     continue
-                                print unicode(translate("DynagenSub", "Local PIX image %s cannot be found, use image %s instead")) \
+                                print translate("DynagenSub", "Local PIX image %s cannot be found, use image %s instead") \
                                 % (unicode(device['image']), image_name)
                                 device['image'] = image_name
-    
+                elif emulator == 'vbox' and (host == globals.GApp.systconf['vbox'].VBoxManager_binding or host == 'localhost') and globals.GApp.systconf['vbox'].enable_VBoxManager:
+                    debugmsg(2, "DynagenSub::open_config(), entered VBoxManager")
+                    globals.GApp.VBoxManager.startVBox(int(controlPort))
+
+                    # Check if this is a relative working directory path and convert to an absolute path if necessary
+                    #"""
+                    if server['workingdir']:
+                        abspath = os.path.join(os.path.dirname(FILENAME), unicode(server['workingdir']))
+                        debugmsg(3, "DynagenSub::open_config(), 'vbox', abspath = %s" % unicode(abspath))
+                        if os.path.exists(abspath):
+                            server['workingdir'] = abspath
+                            debug(unicode("Converting relative working directory path to absolute path: %s") % server['workingdir'])
+
+                    debugmsg(3, "DynagenSub::open_config(), 'vbox', os.path.dirname(FILENAME) = %s" % unicode(os.path.dirname(FILENAME)))
+                    if server['workingdir'] == '.':
+                        server['workingdir'] = os.path.dirname(FILENAME)
+
+                    for subsection in server.sections:
+                        device = server[subsection]
+                        debugmsg(3, "DynagenSub::open_config(), 'vbox', device...")
+                        #debugmsg(3, "DynagenSub::open_config(), device = %s" % str(device))
+                        """  # Disabled whole test, as it fails on multi-host systems, if VBoxManager enabled.
+
+                        # Check if the image path is a relative path
+                        if os.path.exists(device['image']) == False:
+                            abspath = os.path.join(os.path.dirname(FILENAME), unicode(device['image']))
+                            debugmsg(3, "DynagenSub::open_config(), 'vbox', abspath = %s" % str(abspath))
+                            if os.path.exists(abspath):
+                                device['image'] = abspath
+                        #"""
+                        #if device.name == 'VBoxDevice' and device['image']:
+                        #    debugmsg(2, "DynagenSub.py:  if device.name == 'VBoxDevice' and device['image']")
+                        #    #Warning about non-existend VBox Machine disabled.
+                        #    #I decided to skip check on import. Just imported VMs won't run, so user can manually fix them.
+                        #    #VirtualBox will give correct error anyway, on VM start attempt.
+                    #"""
             else:
                 server.host = server.name
+                debugmsg(3, "DynagenSub::open_config(), server.host = %s, server.name = %s" % (unicode(server.host), unicode(server.name)))
                 controlPort = None
                 if ':' in server.host:
-                    (server.host, controlPort) = server.host.split(':')
+                    #(server.host, controlPort) = server.host.split(':')
+                    controlPort = server.host.split(':')[-1]
+                    server.host = self.getHost(server.host)
                 if server['port'] != None:
                     controlPort = server['port']
                 if controlPort == None:
@@ -261,34 +336,44 @@ class DynagenSub(Dynagen):
                 # need to start local hypervisors
                 if (server.host == globals.GApp.systconf['dynamips'].HypervisorManager_binding or server.host == 'localhost') and \
                     globals.GApp.HypervisorManager and globals.GApp.systconf['dynamips'].import_use_HypervisorManager:
+                    debugmsg(2, "DynagenSub::open_config(), entered Dynamips HypervisorManager")
+
+                    # update server.host and server.name to match with Hypervisor Manager Binding configuration,
+                    # having hypervisors using 127.0.0.1 mixed with others using localhost will bring issues ...
+                    if (server.host == 'localhost' and server.host != globals.GApp.systconf['dynamips'].HypervisorManager_binding):
+                        print "Warning: using localhost in your topology file is not recommended"
+#                        server.host = globals.GApp.systconf['dynamips'].HypervisorManager_binding
+#                        server.name = server.host + ':' + controlPort
+
                     debug("Start hypervisor on port: " + str(controlPort))
-                    hypervisor = globals.GApp.HypervisorManager.startNewHypervisor(int(controlPort))
+                    hypervisor = globals.GApp.HypervisorManager.startNewHypervisor(int(controlPort), processcheck=False)
                     globals.GApp.HypervisorManager.waitHypervisor(hypervisor)
-                    
+
                     # Check if this is a relative working directory path and convert to an absolute path if necessary
                     if server['workingdir']:
                         abspath = os.path.join(os.path.dirname(FILENAME), unicode(server['workingdir']))
                         if os.path.exists(abspath):
                             server['workingdir'] = abspath
                             debug(unicode("Converting relative working directory path to absolute path: %s") % server['workingdir'])
-                    
+
                     if server['workingdir'] == '.':
                         server['workingdir'] = os.path.dirname(FILENAME)
-                    
+
 
                     # check if the working directory is accessible, if not find an alternative working directory
-                    if not server['workingdir'] or not os.access(server['workingdir'], os.F_OK):
+                    if not server.has_key('workingdir') or not server['workingdir'] or not os.access(server['workingdir'], os.F_OK):
                         if globals.GApp.workspace.projectWorkdir and os.access(globals.GApp.workspace.projectWorkdir, os.F_OK):
                             workdir = globals.GApp.workspace.projectWorkdir
                         else:
                             workdir = globals.GApp.systconf['dynamips'].workdir
-                        print unicode(translate("DynagenSub", "Local working directory %s cannot be found for hypervisor %s, use working directory %s instead")) \
-                        % (unicode(server['workingdir']), unicode(server.host) + ':' + controlPort, workdir)
+                        print translate("DynagenSub", "Local working directory %s cannot be found for hypervisor %s:%s, use working directory %s instead") \
+                        % (server['workingdir'], server.host, controlPort, workdir)
                         server['workingdir'] = workdir
 
+                    debugmsg(3, ("DynagenSub::open_config(), server.sections = ", server.sections))
                     for subsection in server.sections:
                         device = server[subsection]
-                        
+
                         # check if the IOS image is accessible, if not find an alternative image
                         if device.name in DEVICETUPLE:
                             # Check if this is a relative image path and convert to an absolute path if necessary
@@ -298,7 +383,7 @@ class DynagenSub(Dynagen):
                                 debug(unicode("Converting relative image path to absolute path: %s") % device['image'])
 
                             if not os.access(device['image'], os.F_OK):
-                                
+
                                 selected_images = []
                                 image_to_use = None
                                 for (image, conf) in globals.GApp.iosimages.iteritems():
@@ -306,16 +391,17 @@ class DynagenSub(Dynagen):
                                         selected_images.append(image)
 
                                 if len(selected_images):
-                                    message = "Local IOS image %s\ncannot be found for hypervisor %s\n\nPlease choose an alternative image:" % (unicode(device['image']), unicode(server.host) + ':' + controlPort)
+                                    message = unicode("Local IOS image %s\ncannot be found for hypervisor %s:%s\n\nPlease choose an alternative image:") % (device['image'], server.host, controlPort)
+                                    selected_images.sort()
                                     (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("DynagenSub", "IOS image"),
                                                                                       translate("DynagenSub", message), selected_images, 0, False)
                                     if ok:
                                         image_to_use = unicode(selection)
 
                                 if image_to_use == None and len(selected_images) == 0:
-                                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub', 
-                                                               unicode(translate("IOS image", "IOS image %s cannot be found for hypervisor %s and cannot find an alternative %s image")) 
-                                                                % (device['image'], unicode(server.host) + ':' + controlPort, device.name))
+                                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
+                                                               translate("IOS image", "IOS image %s cannot be found for hypervisor %s:%s and cannot find an alternative %s image")
+                                                                % (device['image'], server.host, controlPort, device.name))
                                     continue
                                 if image_to_use == None and len(selected_images) > 1:
                                     for image in selected_images:
@@ -328,24 +414,25 @@ class DynagenSub(Dynagen):
                                 image_name = globals.GApp.iosimages[image_to_use].filename
                                 ram = globals.GApp.iosimages[image_to_use].default_ram
                                 idlepc = globals.GApp.iosimages[image_to_use].idlepc
-                                print unicode(translate("DynagenSub", "Local IOS image %s cannot be found for hypervisor %s, use image %s instead")) \
-                                % (unicode(device['image']), unicode(server.host) + ':' + controlPort, image_name)
+                                print translate("DynagenSub", "Local IOS image %s cannot be found for hypervisor %s:s, use image %s instead") \
+                                % (device['image'], server.host, controlPort, image_name)
                                 device['image'] = image_name
                                 device['ram'] = ram
                                 device['idlepc'] = idlepc
 
                         # check if the config file is accessible, if not find an alternative config
                         elif device.has_key('cnfg') and device['cnfg']:
-                            
+
                             # Check if this is a relative config path and convert to an absolute path if necessary
+                            debugmsg(3, "DynagenSub::open_config(), device.has_key('cnfg') and device['cnfg']")
                             abspath = os.path.join(os.path.dirname(FILENAME), unicode(device['cnfg']))
                             if os.path.exists(abspath):
                                 device['cnfg'] = abspath
                                 debug(unicode("Converting relative config path to absolute path: %s") % device['cnfg'])
-                            
+
                             if not os.access(device['cnfg'], os.F_OK):
                                 if globals.GApp.workspace.projectConfigs:
-                                    
+
                                     basename =  os.path.basename(device['cnfg'])
                                     if sys.platform.startswith('win') and basename == device['cnfg']:
                                         # basename is the same as the original path, maybe it's an unix/posix path
@@ -355,11 +442,11 @@ class DynagenSub(Dynagen):
                                         # basename is the same as the original path, maybe it's a Windows path
                                         import ntpath
                                         basename = ntpath.basename(device['cnfg'])
-                                    
+
                                     new_config_path = globals.GApp.workspace.projectConfigs + os.sep + basename
-                                    
-                                    print unicode(translate("DynagenSub", "Local configuration %s cannot be found for router %s, use configuration %s instead")) \
-                                    % (unicode(device['cnfg']), unicode(device.name), new_config_path)
+
+                                    print translate("DynagenSub", "Local configuration %s cannot be found for router %s, use configuration %s instead") \
+                                    % (device['cnfg'], device.name, new_config_path)
                                     device['cnfg'] = new_config_path
 
                 current += 1
@@ -367,14 +454,23 @@ class DynagenSub(Dynagen):
         progress.setValue(count)
         progress.deleteLater()
         progress = None
+        debugmsg(2, "DynagenSub::open_config(), returning config")
+        #debugmsg(3, ("config = ", config))
         return config
 
     def getGNS3Data(self):
         """ Returns GNS3 specific data from NET file
         """
+        debugmsg(2, "DynagenSub::getGNS3Data(), returns: %s" % unicode(self.gns3_data))
 
         return self.gns3_data
 
+    def getRpcapMapping(self):
+        """ Returns RPCAP mapping
+        """
+
+        return self.rpcap_mapping
+
     def doerror(self, msg):
         """Print out an error message"""
 
