@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import time, os, sys
@@ -41,7 +41,8 @@ class HypervisorManager(object):
         """ Shutdown all started hypervisors
         """
 
-        self.stopProcHypervisors()
+        for hypervisor in self.hypervisors:
+            hypervisor['proc_instance'].kill()
 
     def setDefaults(self):
         """ Set the default values for the hypervisor manager
@@ -55,51 +56,62 @@ class HypervisorManager(object):
         globals.hypervisor_baseport = dynamips.port
         globals.GApp.dynagen.globaludp = dynamips.baseUDP
 
-    def startNewHypervisor(self, port):
+    def startNewHypervisor(self, port, processcheck=True):
         """ Create a new dynamips process and start it
         """
 
         proc = QtCore.QProcess(globals.GApp.mainWindow)
-        
+
         if self.hypervisor_wd:
             # set the working directory
             proc.setWorkingDirectory(self.hypervisor_wd)
 
-        # test if a hypervisor is already running on this port
-        s = socket(AF_INET, SOCK_STREAM)
-        s.setblocking(0)
-        s.settimeout(300)
-        try:
-            s.connect(('localhost', port))
-            s.close()
+        if processcheck:
+            # test if a hypervisor is already running on this port
+            s = socket(AF_INET, SOCK_STREAM)
+            s.setblocking(0)
+            s.settimeout(300)
+            try:
+                s.connect(('localhost', port))
+                s.close()
 
-            reply = QtGui.QMessageBox.question(globals.GApp.mainWindow, translate("HypervisorManager", "Hypervisor Manager"), unicode(translate("HypervisorManager", "Apparently an hypervisor is already running on port %i, would you like to kill all Dynamips processes?")) % port,
-                                               QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
-            if reply == QtGui.QMessageBox.Yes:       
-                killAll(os.path.basename(self.hypervisor_path))
-                time.sleep(1)
+                reply = QtGui.QMessageBox.question(globals.GApp.mainWindow, translate("HypervisorManager", "Hypervisor Manager"),
+                                                   translate("HypervisorManager", "Apparently an hypervisor is already running on port %i, would you like to kill all Dynamips processes?") % port,
+                                                   QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+                if reply == QtGui.QMessageBox.Yes:
+                    killAll(os.path.basename(self.hypervisor_path))
+                    time.sleep(1)
+                else:
+                    print "Incrementing +100 for base console port, base AUX port, base hypervisor port and base UDP port"
+                    self.baseConsole += 100
+                    if self.baseAUX:
+                        self.baseAUX += 100
+                    globals.hypervisor_baseport += 100
+                    globals.GApp.dynagen.globaludp += 100
+                    port = globals.hypervisor_baseport
 
-            s.connect(('localhost', port))
-            s.close()
-    
-            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("HypervisorManager", "Hypervisor Manager"),
-                                       unicode(translate("HypervisorManager", "A program is still running on port %i, you will have to stop it manually or change port settings")) % port)
+                s.connect(('localhost', port))
+                s.close()
 
-            globals.hypervisor_baseport += 1
-            return None
-        except:
-            s.close()
+                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("HypervisorManager", "Hypervisor Manager"),
+                                           translate("HypervisorManager", "A program is still running on port %i, you will have to stop it manually or change port settings") % port)
+
+                globals.hypervisor_baseport += 1
+                return None
+            except:
+                s.close()
 
         # start dynamips in hypervisor mode (-H)
-        proc.start( self.hypervisor_path ,  ['-H', str(port)])
+        proc.start(self.hypervisor_path,  ['-H', str(port)])
 
         if proc.waitForStarted() == False:
-            QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Hypervisor Manager',  unicode(translate("HypervisorManager", "Can't start Dynamips on port %i")) % port)
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Hypervisor Manager', translate("HypervisorManager", "Can't start Dynamips on port %i") % port)
             return None
 
         hypervisor = {'port': port,
                       'proc_instance': proc,
-                      'load': 0}
+                      'load': 0,
+                      'image_ref': ''}
 
         self.hypervisors.append(hypervisor)
         return hypervisor
@@ -119,8 +131,8 @@ class HypervisorManager(object):
             s.setblocking(0)
             s.settimeout(300)
             if nb == 3:
-                progress = QtGui.QProgressDialog(unicode(translate("HypervisorManager", "Connecting to a hypervisor on port %i ...")) % hypervisor['port'],
-                                                                                                                                        translate("HypervisorManager", "Abort"), 0, count, globals.GApp.mainWindow)
+                progress = QtGui.QProgressDialog(translate("HypervisorManager", "Connecting to a hypervisor on port %i ...") % hypervisor['port'],
+                                                 translate("HypervisorManager", "Abort"), 0, count, globals.GApp.mainWindow)
                 progress.setMinimum(1)
                 progress.setWindowModality(QtCore.Qt.WindowModal)
                 globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 2000)
@@ -146,12 +158,10 @@ class HypervisorManager(object):
             globals.hypervisor_baseport += 1
             time.sleep(0.2)
         else:
-            if last_exception:
-                debug("Hypervisor manager: last exception raised by socket: " + unicode(last_exception))
-            else:
+            if not last_exception:
                 last_exception = 'Unknown problem'
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Hypervisor Manager',
-                                       unicode(translate("HypervisorManager", "Can't connect to the hypervisor on port %i: %s")) % (hypervisor['port'], last_exception))
+                                       translate("HypervisorManager", "Can't connect to the hypervisor on port %i: %s") % (hypervisor['port'], last_exception))
             hypervisor['proc_instance'].close()
             self.hypervisors.remove(hypervisor)
             return False
@@ -168,6 +178,8 @@ class HypervisorManager(object):
         for hypervisor in self.hypervisors:
             if not isinstance(node, IOSRouter) or (isinstance(node, IOSRouter) and hypervisor['load'] + node.default_ram <= globals.GApp.systconf['dynamips'].memory_limit):
                 if isinstance(node, IOSRouter):
+                    if globals.GApp.systconf['dynamips'].allocateHypervisorPerIOS and hypervisor['image_ref'] != node.image_reference:
+                        continue
                     hypervisor['load'] += node.default_ram
                 debug('Hypervisor manager: allocates an already started hypervisor (port: ' + str(hypervisor['port']) + ')')
                 if not globals.GApp.dynagen.dynamips.has_key(globals.GApp.systconf['dynamips'].HypervisorManager_binding + ':' + str(hypervisor['port'])):
@@ -185,16 +197,18 @@ class HypervisorManager(object):
 
         if isinstance(node, IOSRouter):
             hypervisor['load'] = node.default_ram
+            hypervisor['image_ref'] = node.image_reference
+
         # use project workdir in priority
         if globals.GApp.workspace.projectWorkdir:
             if not os.access(globals.GApp.workspace.projectWorkdir, os.F_OK | os.W_OK):
-                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'HypervisorManager', 
-                                          unicode(translate("HypervisorManager", "Working directory %s seems to not exist or be writable, please check")) % globals.GApp.workspace.projectWorkdir)
+                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'HypervisorManager',
+                                          translate("HypervisorManager", "Working directory %s seems to not exist or be writable, please check") % globals.GApp.workspace.projectWorkdir)
             globals.GApp.dynagen.defaults_config['workingdir'] = globals.GApp.workspace.projectWorkdir
         elif self.hypervisor_wd:
             if not os.access(self.hypervisor_wd, os.F_OK | os.W_OK):
-                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'HypervisorManager', 
-                                          unicode(translate("HypervisorManager", "Working directory %s seems to not exist or be writable, please check")) % self.hypervisor_wd)
+                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'HypervisorManager',
+                                          translate("HypervisorManager", "Working directory %s seems to not exist or be writable, please check") % self.hypervisor_wd)
             globals.GApp.dynagen.defaults_config['workingdir'] = self.hypervisor_wd
         try:
             dynamips_hypervisor = globals.GApp.dynagen.create_dynamips_hypervisor(globals.GApp.systconf['dynamips'].HypervisorManager_binding, hypervisor['port'])
@@ -202,7 +216,7 @@ class HypervisorManager(object):
             dynamips_hypervisor = None
         if not dynamips_hypervisor:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Hypervisor Manager',
-                                        unicode(translate("HypervisorManager", "Can't set up hypervisor on port %i, please check the settings (writable working directory ...)")) % hypervisor['port'])
+                                       translate("HypervisorManager", "Can't set up hypervisor on port %i, please check the settings (writable working directory ...)") % hypervisor['port'])
             if globals.GApp.dynagen.dynamips.has_key(globals.GApp.systconf['dynamips'].HypervisorManager_binding + ':' + str(hypervisor['port'])):
                 del globals.GApp.dynagen.dynamips[globals.GApp.systconf['dynamips'].HypervisorManager_binding + ':' + str(hypervisor['port'])]
             hypervisor['proc_instance'].close()
@@ -236,7 +250,7 @@ class HypervisorManager(object):
                 if hypervisor['load'] <= 0:
                     hypervisor['load'] = 0
                 break
-            
+
     def changeHypervisorLoad(self, node, port, old_default_ram):
         """ Change the hypervisor RAM load for a given node
         """
@@ -278,6 +292,8 @@ class HypervisorManager(object):
                     continue
         for hypervisor in self.hypervisors:
             debug("Hypervisor manager: close hypervisor on port " + str(hypervisor['port']))
+            hypervisor['proc_instance'].terminate()
+            time.sleep(0.5)
             hypervisor['proc_instance'].close()
             hypervisor['proc_instance'] = None
         self.hypervisors = []
