@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import sys, os, re
@@ -26,6 +26,7 @@ from GNS3.Ui.Form_IOSDialog import Ui_IOSDialog
 from GNS3.Utils import fileBrowser, translate, testOpenFile
 from GNS3.Config.Objects import iosImageConf, hypervisorConf
 from GNS3.Node.IOSRouter import IOSRouter
+from GNS3.Uncompress import isIOScompressed, uncompressIOS
 
 # known platforms and corresponding chassis
 PLATFORMS = {
@@ -69,7 +70,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         self.connect(self.treeWidgetIOSimages,  QtCore.SIGNAL('itemSelectionChanged()'),  self.slotIOSSelectionChanged)
         self.connect(self.treeWidgetHypervisor, QtCore.SIGNAL('itemSelectionChanged()'),  self.slotHypervisorSelectionChanged)
         self.connect(self.labelCheckRAM,  QtCore.SIGNAL('linkActivated(const QString &)'), self.slotCheckRAMrequirement)
-        
+
         # insert known platforms
         self.comboBoxPlatform.insertItems(0, PLATFORMS.keys())
 
@@ -79,6 +80,10 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         self.treeWidgetHypervisor.sortByColumn(0, QtCore.Qt.AscendingOrder)
         self.treeWidgetHypervisor.setSortingEnabled(True)
 
+        # If IOS path exists, baseconfig.txt should be inside
+        if os.path.exists(globals.GApp.systconf['general'].ios_path + os.sep + 'baseconfig.txt'):
+            self.lineEditBaseConfig.setText(globals.GApp.systconf['general'].ios_path + os.sep + 'baseconfig.txt')
+
         # reload saved infos
         self._reloadInfos()
 
@@ -168,9 +173,9 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
             path = os.path.normpath(path[0])
             # test if we can open it
             if not testOpenFile(path):
-                QtGui.QMessageBox.critical(self, translate("IOSDialog", "IOS Configuration"), unicode(translate("IOSDialog", "Can't open file: %s")) % path)
+                QtGui.QMessageBox.critical(self, translate("IOSDialog", "IOS Configuration"), translate("IOSDialog", "Can't open file: %s") % path)
                 return
-            
+
             if sys.platform.startswith('win'):
                 try:
                     path.encode('ascii')
@@ -178,14 +183,37 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
                     QtGui.QMessageBox.warning(self, translate("IOSDialog", "IOS Configuration"), translate("IOSDialog", "The path you have selected should contains only ascii (English) characters. Dynamips (Cygwin DLL) doesn't support unicode on Windows!"))
 
             self.lineEditIOSImage.clear()
+
+            try:
+                if isIOScompressed(path):
+                    if path.endswith('.bin'):
+                        extracted_ios = path.rsplit(".bin")[0] + '.image'
+                    else:
+                        extracted_ios = path + '.image'
+                    reply = QtGui.QMessageBox.question(self, translate("IOSDialog", "IOS Image"),
+                                                       translate("IOSDialog", "The IOS image is compressed. Would you like to uncompress it? Yes will create %s") % os.path.basename(extracted_ios),
+                                                       QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+                    if reply == QtGui.QMessageBox.Yes:
+                        if os.path.exists(extracted_ios):
+                            QtGui.QMessageBox.warning(self, translate("IOSDialog", "IOS Image"), translate("IOSDialog", "%s already exists, let's use it") % extracted_ios)
+                            path = extracted_ios
+                        else:
+                            try:
+                                uncompressIOS(path, extracted_ios)
+                                path = extracted_ios
+                            except:
+                                QtGui.QMessageBox.critical(self, translate("IOSDialog", "IOS Image"), translate("IOSDialog", "Sorry, impossible to extract the IOS image"))
+            except:
+                pass
+
             self.lineEditIOSImage.setText(path)
-            
+
             # basename doesn't work on Unix with Windows paths, so let's use this little trick
             image = path
             if not sys.platform.startswith('win') and image[1] == ":":
                 image = image[2:]
                 image = image.replace("\\", "/")
-                
+
             # try to guess the platform
             platform = self._getIOSplatform(os.path.basename(image))
             if platform == '2600':
@@ -216,23 +244,23 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
     def slotBaseConfig(self):
         """ Get an base config file from the file system
         """
-        
-        # get the path to the ios image
-        path = fileBrowser(translate("IOSDialog", "Select a Base configuration file"),  directory='.', parent=self).getFile()
+
+        # get the path to the ios configuration
+        path = fileBrowser(translate("IOSDialog", "Select a Base configuration file"),  directory=globals.GApp.systconf['general'].ios_path, parent=self).getFile()
 
         if path != None and path[0] != '':
             path = os.path.normpath(path[0])
             # test if we can open it
             if not testOpenFile(path):
-                QtGui.QMessageBox.critical(self, translate("IOSDialog", "IOS Configuration"), unicode(translate("IOSDialog", "Can't open file: %s")) % path)
+                QtGui.QMessageBox.critical(self, translate("IOSDialog", "IOS Configuration"), translate("IOSDialog", "Can't open file: %s") % path)
                 return
-            
+
             if sys.platform.startswith('win'):
                 try:
                     path.encode('ascii')
                 except:
                     QtGui.QMessageBox.warning(self, translate("IOSDialog", "IOS Configuration"), translate("IOSDialog", "The path you have selected should contains only ascii (English) characters. Dynamips (Cygwin DLL) doesn't support unicode on Windows!"))
-            
+
             self.lineEditBaseConfig.clear()
             self.lineEditBaseConfig.setText(path)
 
@@ -240,8 +268,8 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         """ Save an IOS image and all his settings
         """
 
-        imagename = unicode(self.lineEditIOSImage.text())
-        
+        imagename = unicode(self.lineEditIOSImage.text(), 'utf-8', errors='replace')
+
         if not imagename:
             return
 
@@ -249,7 +277,12 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         if idlepc and not re.search(r"""^0x[0-9a-fA-F]{8}$""", idlepc):
             QtGui.QMessageBox.critical(self, translate("IOSDialog", "IOS Configuration"), translate("IOSDialog", "IDLE PC not valid (format required: 0xhhhhhhhh)"))
             return
-        
+
+        if not idlepc:
+            self.label_IdlePCWarning.setText('<font color="red">' + translate("IOSDialog", "Warning: IDLE PC will have to be configured! <a href='http://www.gns3.net/gns3-simplest-topology' >Find out why and how</a>")  + '</font>')
+        else:
+            self.label_IdlePCWarning.setText('')
+
         hypervisors = []
         if self.checkBoxIntegratedHypervisor.checkState() == QtCore.Qt.Unchecked:
             # external hypervisor, don't use the hypervisor manager
@@ -262,12 +295,12 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
                 # get the selected hypervisor
                 if len(items) > 1:
                     for item in items:
-                        selected_hypervisor = unicode(item.text())
+                        selected_hypervisor = unicode(item.text(), 'utf-8', errors='replace')
                         hypervisor = globals.GApp.hypervisors[selected_hypervisor]
                         hypervisors.append(hypervisor.host + ':' + str(hypervisor.port))
                     imagekey = 'load-balanced-on-external-hypervisors:' + imagename
                 else:
-                    selected_hypervisor = unicode(items[0].text())
+                    selected_hypervisor = unicode(items[0].text(), 'utf-8', errors='replace')
                     hypervisor = globals.GApp.hypervisors[selected_hypervisor]
                     hypervisors.append(hypervisor.host + ':' + str(hypervisor.port))
                     imagekey = hypervisor.host + ':' + imagename
@@ -297,7 +330,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         conf.id = globals.GApp.iosimages_ids
         globals.GApp.iosimages_ids += 1
         conf.filename = imagename
-        conf.baseconfig = unicode(self.lineEditBaseConfig.text())
+        conf.baseconfig = unicode(self.lineEditBaseConfig.text(), 'utf-8', errors='replace')
         conf.platform = str(self.comboBoxPlatform.currentText())
         conf.chassis = str(self.comboBoxChassis.currentText())
         conf.idlepc = idlepc
@@ -333,7 +366,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         item = self.treeWidgetIOSimages.currentItem()
         if (item != None):
             self.treeWidgetIOSimages.takeTopLevelItem(self.treeWidgetIOSimages.indexOfTopLevelItem(item))
-            image = unicode(item.text(0))
+            image = unicode(item.text(0), 'utf-8', errors='replace')
             del globals.GApp.iosimages[image]
 
     def slotIOSSelectionChanged(self):
@@ -342,8 +375,8 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
 
         item = self.treeWidgetIOSimages.currentItem()
         if item != None:
-            self.pushButtonDeleteIOS.setEnabled(True) 
-            imagekey = unicode(item.text(0))
+            self.pushButtonDeleteIOS.setEnabled(True)
+            imagekey = unicode(item.text(0), 'utf-8', errors='replace')
             self.selectionChanged = False
             if globals.GApp.iosimages.has_key(imagekey):
                 conf = globals.GApp.iosimages[imagekey]
@@ -368,7 +401,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
                         if items:
                             items[0].setSelected(True)
                 else:
-                    self.checkBoxIntegratedHypervisor.setCheckState(QtCore.Qt.Checked) 
+                    self.checkBoxIntegratedHypervisor.setCheckState(QtCore.Qt.Checked)
         else:
             self.pushButtonDeleteIOS.setEnabled(False)
 
@@ -376,7 +409,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         """ Check for minimum RAM requirement
         """
 
-        image_file = unicode(self.lineEditIOSImage.text())
+        image_file = unicode(self.lineEditIOSImage.text(), 'utf-8', errors='replace')
         if not image_file:
             QtGui.QMessageBox.warning(self, translate("IOSDialog", "IOS Configuration"), translate("IOSDialog", "Image file box is empty"))
             return
@@ -407,7 +440,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         if path:
             self.lineEditWorkingDir.clear()
             self.lineEditWorkingDir.setText(os.path.normpath(path))
-            
+
             if sys.platform.startswith('win'):
                 try:
                     path.encode('ascii')
@@ -418,9 +451,9 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         """ Save a hypervisor to the hypervisors list
         """
 
-        hypervisor_host = unicode(self.lineEditHost.text())
+        hypervisor_host = unicode(self.lineEditHost.text(), 'utf-8', errors='replace')
         hypervisor_port = str(self.spinBoxHypervisorPort.value())
-        working_dir = unicode(self.lineEditWorkingDir.text())
+        working_dir = unicode(self.lineEditWorkingDir.text(), 'utf-8', errors='replace')
         baseudp = self.spinBoxBaseUDP.value()
 
         if (hypervisor_host != '' and hypervisor_port != ''):
@@ -452,6 +485,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
             conf.workdir = working_dir
             conf.baseUDP = baseudp
             conf.baseConsole = self.spinBoxBaseConsole.value()
+            conf.baseAUX = self.spinBoxBaseAUX.value()
             self.spinBoxBaseUDP.setValue(conf.baseUDP + 100)
             self.spinBoxBaseConsole.setValue(conf.baseConsole + 10)
             globals.GApp.hypervisors[hypervisorkey] = conf
@@ -477,7 +511,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
         item = self.treeWidgetHypervisor.currentItem()
         if item != None:
             self.pushButtonDeleteHypervisor.setEnabled(True)
-            hypervisor_key = unicode(item.text(0))
+            hypervisor_key = unicode(item.text(0), 'utf-8', errors='replace')
             if globals.GApp.hypervisors.has_key(hypervisor_key):
                 conf = globals.GApp.hypervisors[hypervisor_key]
                 self.lineEditHost.setText(conf.host)
@@ -485,6 +519,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
                 self.lineEditWorkingDir.setText(conf.workdir)
                 self.spinBoxBaseUDP.setValue(conf.baseUDP)
                 self.spinBoxBaseConsole.setValue(conf.baseConsole)
+                self.spinBoxBaseAUX.setValue(conf.baseAUX)
         else:
             self.pushButtonDeleteHypervisor.setEnabled(False)
 
