@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
+# Copyright (C) 2007-2011 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,14 +16,24 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
+#debuglevel: 0=disabled, 1=default, 2=debug, 3=deep debug
+debuglevel = 0
+
+def debugmsg(level, message):
+    if debuglevel == 0:
+        return
+    if debuglevel >= level:
+        print message
+
 import os, re, random, base64, traceback
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynagen as dynagen_namespace
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Dynagen.qemu_lib as qlib
+import GNS3.Dynagen.dynagen_vbox_lib as vboxlib
 from GNS3.Globals.Symbols import SYMBOLS
 from GNS3.Utils import translate, debug, error
 from PyQt4 import QtGui, QtCore, QtSvg
@@ -42,6 +52,7 @@ from GNS3.Node.ETHSW import ETHSW, init_ethsw_id
 from GNS3.Node.FRSW import init_frsw_id
 from GNS3.Node.Cloud import Cloud, init_cloud_id
 from GNS3.Node.AnyEmuDevice import init_emu_id, AnyEmuDevice
+from GNS3.Node.AnyVBoxEmuDevice import init_vbox_emu_id, AnyVBoxEmuDevice
 
 router_hostname_re = re.compile(r"""^R([0-9]+)""")
 ethsw_hostname_re = re.compile(r"""^SW([0-9]+)""")
@@ -49,7 +60,8 @@ frsw_hostname_re = re.compile(r"""^FR([0-9]+)""")
 atmsw_hostname_re = re.compile(r"""^ATM([0-9]+)""")
 atmbr_hostname_re = re.compile(r"""^BR([0-9]+)""")
 cloud_hostname_re = re.compile(r"""^C([0-9]+)""")
-emu_hostname_re = re.compile(r"""^[FW|JUNOS|ASA|IDS|QEMU]([0-9]+)""")
+emu_hostname_re = re.compile(r"""^[PIX|JUNOS|ASA|IDS|QEMU]([0-9]+)""")
+vbox_emu_hostname_re = re.compile(r"""^[VBOX]([0-9]+)""")
 decorative_hostname_re = re.compile(r"""^N([0-9]+)""")
 
 class NETFile(object):
@@ -57,14 +69,16 @@ class NETFile(object):
     """
 
     def __init__(self):
-
+        debugmsg(2, "NETFile::__init__()")
         self.dynagen = globals.GApp.dynagen
+
         self.connection2cloud = {}
         self.decorative_node_connections = {}
 
     def add_in_connection_list(self, connection_data, connection_list):
         """ Record the connection in connection_list
         """
+        debugmsg(2, "NETFile::add_in_connection_list()")
 
         (source_device, source_interface, destination_device, destination_interface) = connection_data
         # don't want to record bidirectionnal connections
@@ -77,6 +91,7 @@ class NETFile(object):
     def populate_connection_list_for_router(self, device, connection_list):
         """ Add router connections in connection_list
         """
+        debugmsg(2, "NETFile::populate_connection_list_for_router()")
 
         for adapter in device.slot:
             if adapter:
@@ -105,24 +120,28 @@ class NETFile(object):
                                                                                                                                                                             connection_list)
                                 elif isinstance(remote_device, lib.FRSW) or isinstance(remote_device, lib.ATMSW) or isinstance(remote_device, lib.ETHSW) or isinstance(remote_device, lib.ATMBR):
                                     connection_list.append((device.name, source_interface, remote_device.name, str(remote_port)))
-                                elif isinstance(remote_device, qlib.AnyEmuDevice):
+                                elif isinstance(remote_device, qlib.AnyEmuDevice) or isinstance(remote_device, vboxlib.AnyVBoxEmuDevice):
                                     connection_list.append((device.name, source_interface, remote_device.name, remote_adapter + str(remote_port)))
 
     def populate_connection_list_for_emulated_device(self, device, connection_list):
         """ Add emulated device connections in connection_list
         """
+        debugmsg(2, "NETFile::populate_connection_list_for_emulated_device()")
 
         for port in device.nios:
             if device.nios[port] != None:
                 (remote_device, remote_adapter, remote_port) = lib.get_reverse_udp_nio(device.nios[port])
                 if isinstance(remote_device, qlib.AnyEmuDevice):
                     self.add_in_connection_list((device.name, 'e' + str(port), remote_device.name, remote_adapter + str(remote_port)), connection_list)
+                elif isinstance(remote_device, vboxlib.AnyVBoxEmuDevice):
+                    self.add_in_connection_list((device.name, 'e' + str(port), remote_device.name, remote_adapter + str(remote_port)), connection_list)
                 elif isinstance(remote_device, lib.ETHSW):
                     connection_list.append((device.name, 'e' + str(port), remote_device.name, str(remote_port)))
-                    
+
     def populate_connection_list_for_emulated_switch(self, device, connection_list):
         """ Add emulated switch connections in connection_list
         """
+        debugmsg(2, "NETFile::populate_connection_list_for_emulated_switch()")
 
         if isinstance(device, lib.ETHSW):
             keys = device.mapping.keys()
@@ -133,7 +152,7 @@ class NETFile(object):
                     (remote_device, remote_adapter, remote_port) = lib.get_reverse_udp_nio(nio_port)
                     if isinstance(remote_device, lib.ETHSW):
                         self.add_in_connection_list((device.name, str(port), remote_device.name, str(remote_port)), connection_list)
-        
+
         if isinstance(device, lib.FRSW):
             keys = device.pvcs.keys()
             for (port, dlci) in keys:
@@ -152,7 +171,7 @@ class NETFile(object):
                     (remote_device, remote_adapter, remote_port) = lib.get_reverse_udp_nio(nio_port)
                     if isinstance(remote_device, lib.ATMSW) or isinstance(remote_device, lib.ATMBR):
                         self.add_in_connection_list((device.name, str(port), remote_device.name, str(remote_port)), connection_list)
-                        
+
         if isinstance(device, lib.ATMBR):
             keys = device.mapping.keys()
             for port in keys:
@@ -160,21 +179,25 @@ class NETFile(object):
                 if nio_port:
                     (remote_device, remote_adapter, remote_port) = lib.get_reverse_udp_nio(nio_port)
                     if isinstance(remote_device, lib.ATMSW) or isinstance(remote_device, lib.ATMBR):
-                        self.add_in_connection_list((device.name, str(port), remote_device.name, str(remote_port)), connection_list)       
+                        self.add_in_connection_list((device.name, str(port), remote_device.name, str(remote_port)), connection_list)
 
     def create_node(self, device, default_symbol_name, running_config_name):
         """ Create a new node
         """
+        debugmsg(2, "****    NETFile::create_node(%s, %s, %s)" % (unicode(device), unicode(default_symbol_name), unicode(running_config_name)))
 
         symbol_name = x = y = z = hx = hy = None
         config = None
-        if isinstance(device, qlib.AnyEmuDevice) and self.dynagen.globalconfig['qemu ' + device.dynamips.host +':' + str(device.dynamips.port)].has_key(running_config_name):
+        if   isinstance(device, qlib.AnyEmuDevice)        and self.dynagen.globalconfig['qemu ' + device.dynamips.host +':' + str(device.dynamips.port)].has_key(running_config_name):
             config = self.dynagen.globalconfig['qemu ' + device.dynamips.host +':' + str(device.dynamips.port)][running_config_name]
+        elif isinstance(device, vboxlib.AnyVBoxEmuDevice) and self.dynagen.globalconfig['vbox ' + device.dynamips.host +':' + str(device.dynamips.port)].has_key(running_config_name):
+            config = self.dynagen.globalconfig['vbox ' + device.dynamips.host +':' + str(device.dynamips.port)][running_config_name]
         elif self.dynagen.globalconfig.has_key(device.dynamips.host +':' + str(device.dynamips.port)) and \
             self.dynagen.globalconfig[device.dynamips.host +':' + str(device.dynamips.port)].has_key(running_config_name):
             config = self.dynagen.globalconfig[device.dynamips.host +':' + str(device.dynamips.port)][running_config_name]
         elif self.dynagen.globalconfig.has_key(device.dynamips.host) and self.dynagen.globalconfig[device.dynamips.host].has_key(running_config_name):
             config = self.dynagen.globalconfig[device.dynamips.host][running_config_name]
+        #print "config = %s" % str(config)
 
         if config:
             if config.has_key('x'):
@@ -190,6 +213,7 @@ class NETFile(object):
             if config.has_key('symbol'):
                 symbol_name = config['symbol']
 
+        #print "symbol_name = %s" % str(symbol_name)
         node = None
         if symbol_name:
             for item in SYMBOLS:
@@ -205,6 +229,7 @@ class NETFile(object):
                             node.default_symbol = False
                             break
                     break
+        debugmsg(3, "NETFile.py, node = %s" % unicode(node))
 
         if not node:
             # symbol name not found, use default one
@@ -212,8 +237,8 @@ class NETFile(object):
             if not symbol_name or not globals.GApp.scene.renders.has_key(symbol_name):
                 symbol_name = default_symbol_name
                 default_symbol = True
-    
-    
+
+            debugmsg(3, "NETFile.py, symbol_name = %s" % unicode(symbol_name))
             for item in SYMBOLS:
                 if item['name'] == symbol_name:
                     renders = globals.GApp.scene.renders[symbol_name]
@@ -222,11 +247,11 @@ class NETFile(object):
                     if not default_symbol:
                         node.default_symbol = False
                     break
-        
-        if not node:    
+        debugmsg(3, "NETFile.py, node = %s" % unicode(node))
+        if not node:
             return None
-             
-        node.set_hostname(device.name)       
+
+        node.set_hostname(device.name)
         if x == None:
             x = random.uniform(-200, 200)
         if y == None:
@@ -245,6 +270,7 @@ class NETFile(object):
     def record_image(self, device):
         """ Record an image and all its settings in GNS3
         """
+        debugmsg(2, "NETFile::record_image()")
 
         conf_image = iosImageConf()
         conf_image.id = globals.GApp.iosimages_ids
@@ -283,6 +309,7 @@ class NETFile(object):
     def configure_node(self, node, device):
         """ Configure a node
         """
+        debugmsg(2, "NETFile::configure_node()")
 
         if isinstance(device, lib.Router):
             if (device.dynamips.host == globals.GApp.systconf['dynamips'].HypervisorManager_binding or device.dynamips.host == 'localhost') and \
@@ -300,10 +327,11 @@ class NETFile(object):
         node.set_dynagen_device(device)
         device.dynamips.configchange = True
         return True
-    
+
     def add_connection(self, connection):
         """ Add a connection
         """
+        debugmsg(2, "NETFile::add_connection()")
 
         debug('Add connection ' + str(connection))
         (source_name, source_interface, destination_name, destination_interface) = connection
@@ -320,12 +348,12 @@ class NETFile(object):
 
         globals.GApp.topology.recordLink(srcid, source_interface, dstid, destination_interface, src_node, dst_node)
 
-        if not isinstance(src_node, IOSRouter) and not isinstance(src_node, AnyEmuDevice):
+        if not isinstance(src_node, IOSRouter) and not isinstance(src_node, AnyEmuDevice) and not isinstance(src_node, AnyVBoxEmuDevice):
             if not isinstance(src_node,Cloud) and not src_node.hypervisor:
                 src_node.get_dynagen_device()
             src_node.startupInterfaces()
             src_node.state = 'running'
-        if not isinstance(dst_node, IOSRouter) and not isinstance(dst_node, AnyEmuDevice):
+        if not isinstance(dst_node, IOSRouter) and not isinstance(dst_node, AnyEmuDevice) and not isinstance(dst_node, AnyVBoxEmuDevice):
             if not isinstance(dst_node,Cloud) and not dst_node.hypervisor:
                 dst_node.get_dynagen_device()
             dst_node.startupInterfaces()
@@ -334,6 +362,7 @@ class NETFile(object):
     def create_cloud(self, nio, source_device, source_interface):
         """ Create a cloud (used for NIO connections)
         """
+        debugmsg(2, "NETFile::create_cloud()")
 
         nio = nio.lower()
         if self.connection2cloud.has_key((source_device, source_interface, nio)):
@@ -358,6 +387,7 @@ class NETFile(object):
     def apply_gns3_data(self):
         """ Apply specific GNS3 data
         """
+        debugmsg(2, "NETFile::apply_gns3_data()")
 
         max_cloud_id = -1
         max_decorative_id = -1
@@ -376,16 +406,32 @@ class NETFile(object):
                     continue
                 if devtype.lower() == 'cloud':
                     default_symbol = True
-                    if gns3data[section].has_key('symbol') and globals.GApp.scene.renders.has_key(gns3data[section]['symbol']):
-                        symbol = gns3data[section]['symbol']
-                        default_symbol = False
-                    else:
-                        symbol = 'Cloud'
-                    renders = globals.GApp.scene.renders[symbol]
-                    cloud = Cloud(renders['normal'], renders['selected'])
+                    symbol_name = None
+                    if gns3data[section].has_key('symbol') and gns3data[section]['symbol']:
+                        symbol_name = gns3data[section]['symbol']
+
+                        symbol_resources = QtCore.QResource(":/symbols")
+                        for symbol in symbol_resources.children():
+                            symbol = str(symbol)
+                            if symbol.startswith(symbol_name):
+                                normal_renderer = QtSvg.QSvgRenderer(':/symbols/' + symbol_name + '.normal.svg')
+                                select_renderer = QtSvg.QSvgRenderer(':/symbols/' + symbol_name + '.selected.svg')
+                                default_symbol = False
+                                break
+
+                    if default_symbol:
+                        if not symbol_name or not globals.GApp.scene.renders.has_key(symbol_name):
+                            symbol_name = 'Cloud'
+                        else:
+                            default_symbol = False
+                        normal_renderer = globals.GApp.scene.renders[symbol_name]['normal']
+                        select_renderer = globals.GApp.scene.renders[symbol_name]['selected']
+
+                    cloud = Cloud(normal_renderer, select_renderer)
                     config = {}
                     config['nios'] = []
-                    cloud.type = symbol
+                    config['rpcap_mapping'] = dict(self.dynagen.getRpcapMapping())
+                    cloud.type = symbol_name
                     if not default_symbol:
                         cloud.default_symbol = False
                     cloud.hostname = unicode(hostname)
@@ -424,24 +470,28 @@ class NETFile(object):
                     note_object.setPos(float(gns3data[section]['x']), float(gns3data[section]['y']))
                     if gns3data[section].has_key('z'):
                         note_object.setZValue(float(gns3data[section]['z']))
+                        if note_object.zValue() < 0:
+                            # object on background layer, user cannot select it and move it.
+                            note_object.setFlag(note_object.ItemIsSelectable, False)
+                            note_object.setFlag(note_object.ItemIsMovable, False)
                     if gns3data[section].has_key('font'):
                         font = QtGui.QFont()
                         if font.fromString(gns3data[section]['font'][1:-1]):
                             note_object.setFont(font)
                         else:
-                            print unicode(translate("NETFile", "Cannot load font: %s")) % gns3data[section]['font']
+                            print translate("NETFile", "Cannot load font: %s") % gns3data[section]['font']
                     if gns3data[section].has_key('rotate'):
                         note_object.rotation = int(gns3data[section]['rotate'])
                         note_object.rotate(note_object.rotation)
                     if gns3data[section].has_key('color'):
                         note_object.setDefaultTextColor(QtGui.QColor(gns3data[section]['color'][1:-1]))
-                        
+
                     # this is an interface label, save it in a dict to be used later ...
                     if gns3data[section].has_key('interface'):
                         globals.interfaceLabels[gns3data[section]['interface']] = note_object
                     else:
                         globals.GApp.topology.addItem(note_object)
-                    
+
                 if devtype.lower() == 'shape':
                     if gns3data[section]['type'] == 'rectangle':
                         size = QtCore.QSizeF(float(gns3data[section]['width']), float(gns3data[section]['height']))
@@ -451,9 +501,13 @@ class NETFile(object):
                         size = QtCore.QSizeF(float(gns3data[section]['width']), float(gns3data[section]['height']))
                         pos = QtCore.QPointF(float(gns3data[section]['x']), float(gns3data[section]['y']))
                         shape_object = Ellipse(pos, size)
-    
+
                     if gns3data[section].has_key('z'):
                         shape_object.setZValue(float(gns3data[section]['z']))
+                        if shape_object.zValue() < 0:
+                            # object on background layer, user cannot select it and move it.
+                            shape_object.setFlag(shape_object.ItemIsSelectable, False)
+                            shape_object.setFlag(shape_object.ItemIsMovable, False)
                     if gns3data[section].has_key('rotate'):
                         shape_object.rotation = int(gns3data[section]['rotate'])
                         shape_object.rotate(shape_object.rotation)
@@ -469,10 +523,10 @@ class NETFile(object):
                         pen.setStyle(QtCore.Qt.PenStyle(int(gns3data[section]['border_style'])))
                     shape_object.setPen(pen)
                     globals.GApp.topology.addItem(shape_object)
-                    
+
                 if devtype.lower() == 'pixmap':
                     pixmap_path = unicode(gns3data[section]['path'])
-                    
+
                     # Check if this is a relative pixmap path and convert to an absolute path if necessary
                     abspath = os.path.join(os.path.dirname(self.dynagen.filename), pixmap_path)
                     if os.path.exists(abspath):
@@ -483,18 +537,22 @@ class NETFile(object):
                     if not pixmap_image.isNull():
                         pixmap_object= Pixmap(pixmap_image, pixmap_path)
                     else:
-                        print unicode(translate("NETFile", "Cannot load image: %s")) % pixmap_path
+                        print translate("NETFile", "Cannot load image: %s") % pixmap_path
                         continue
                     pixmap_object.setPos(float(gns3data[section]['x']), float(gns3data[section]['y']))
                     if gns3data[section].has_key('z'):
                         pixmap_object.setZValue(float(gns3data[section]['z']))
+                        if pixmap_object.zValue() < 0:
+                            # object on background layer, user cannot select it and move it.
+                            pixmap_object.setFlag(pixmap_object.ItemIsSelectable, False)
+                            pixmap_object.setFlag(pixmap_object.ItemIsMovable, False)
                     globals.GApp.topology.addItem(pixmap_object)
 
                 if devtype.lower() == 'node':
                     hostname = unicode(hostname)
                     symbol = unicode(gns3data[section]['symbol'])
                     if not globals.GApp.scene.renders.has_key(symbol):
-                        print unicode(translate("NETFile", "%s: cannot find %s symbol, please check this symbol is in your node list and reload the .net file")) % (hostname, symbol)
+                        print translate("NETFile", "%s: cannot find %s symbol, please check this symbol is in your node list and reload the .net file") % (hostname, symbol)
                         continue
                     renders = globals.GApp.scene.renders[symbol]
                     decorative_node = DecorativeNode(renders['normal'], renders['selected'])
@@ -530,14 +588,15 @@ class NETFile(object):
             init_cloud_id(max_cloud_id + 1)
         if max_decorative_id != -1:
             init_decoration_id(max_decorative_id + 1)
-            
+
         if len(globals.interfaceLabels):
             globals.GApp.workspace.flg_showOnlySavedInterfaceNames = True
 
     def apply_decorative_node_connections(self):
         """ Create GUI connections for decorative nodes
         """
-        
+        debugmsg(2, "NETFile::apply_decorative_node_connections()")
+
         for (connection, local_device) in self.decorative_node_connections.iteritems():
             (remote_device, remote_interface, local_interface) = connection
             if isinstance(remote_device, IOSRouter):
@@ -549,6 +608,7 @@ class NETFile(object):
     def import_net_file(self, path):
         """ Import a .net file
         """
+        debugmsg(2, "NETFile::import_net_file(%s)" % unicode(path))
 
         if globals.GApp.systconf['dynamips'].import_use_HypervisorManager and globals.GApp.systconf['dynamips'].path == '':
             QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("NETFile", "Save"), translate("NETFile", "Please configure the path to Dynamips"))
@@ -557,17 +617,25 @@ class NETFile(object):
         globals.GApp.workspace.clear()
         dynagen_namespace.CONFIGSPECPATH = []
         dir = os.path.dirname(dynagen_namespace.__file__)
+        debugmsg(3, "NETFile::import_net_file(),    os.path.dirname(dynagen_namespace.__file__) = %s" % unicode(dir))
         dynagen_namespace.CONFIGSPECPATH.append(dir)
         try:
+            debugmsg(3, "NETFile.py: import_config, try: path = %s" % unicode(path))
             dynagen_namespace.FILENAME = path
+            debugmsg(3, "NETFile.py: import_config, try: dynagen.import_config")
             self.dynagen.import_config(path)
+            debugmsg(3, "NETFile.py: import_config, try: QtGui.QSplashScreen()")
             splash = QtGui.QSplashScreen(QtGui.QPixmap(":images/logo_gns3_splash.png"))
             splash.show()
             splash.showMessage(translate("NETFile", "Please wait while importing the topology"))
+            debugmsg(3, "NETFile.py: import_config, try: GApp.processEvents")
             globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
+            debugmsg(3, "NETFile.py: import_config, dynagen.ghosting()")
             self.dynagen.ghosting()
-            self.dynagen.jitsharing()
+            if globals.GApp.systconf['dynamips'].jitsharing:
+                self.dynagen.jitsharing()
         except lib.DynamipsError, msg:
+            debugmsg(1, ("ADEBUG: NETFile.py: lib.DynamipsError: ", msg))
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("NETFile", "Dynamips error"),  unicode(msg))
             globals.GApp.workspace.projectFile = None
             globals.GApp.workspace.setWindowTitle("GNS3")
@@ -580,6 +648,7 @@ class NETFile(object):
             globals.GApp.workspace.clear()
             return
         except Exception, ex:
+            debugmsg(1, ("ADEBUG: NETFile.py: Exception detected: ", ex))
             logfile = open('dynagen_exception.log','a')
             traceback.print_exc(file=logfile)
             logfile.close()
@@ -603,12 +672,13 @@ class NETFile(object):
         max_atmsw_id = -1
         max_atmbr_id = -1
         max_emu_id = -1
+        max_vbox_emu_id = -1
         for (devicename, device) in self.dynagen.devices.iteritems():
 
             if isinstance(device,  lib.Bridge):
                 translate("NETFile", "Warning: GNS3 doesn't yet support lan statements, ignore it")
                 continue
-        
+
             if isinstance(device, lib.Router):
                 platform = device.model
                 # dynamips lib doesn't return c3700, force platform
@@ -747,7 +817,7 @@ class NETFile(object):
                     if id > max_atmbr_id:
                         max_atmbr_id = id
 
-            elif isinstance(device, qlib.AnyEmuDevice):
+            elif isinstance(device, qlib.AnyEmuDevice) or isinstance(device, vboxlib.AnyVBoxEmuDevice):
 
                 node = self.create_node(device, device._ufd_machine, device.gen_cfg_name())
                 assert(node)
@@ -758,8 +828,10 @@ class NETFile(object):
                 match_obj = emu_hostname_re.match(node.hostname)
                 if match_obj:
                     id = int(match_obj.group(1))
-                    if id > max_emu_id:
+                    if isinstance(device, qlib.AnyEmuDevice) and (id > max_emu_id):
                         max_emu_id = id
+                    if isinstance(device, vboxlib.AnyVBoxEmuDevice) and (id > max_vbox_emu_id):
+                        max_vbox_emu_id = id
 
             globals.GApp.topology.addItem(node)
 
@@ -776,6 +848,8 @@ class NETFile(object):
             init_atmbr_id(max_atmbr_id + 1)
         if max_emu_id != -1:
             init_emu_id(max_emu_id + 1)
+        if max_vbox_emu_id != -1:
+            init_vbox_emu_id(max_vbox_emu_id + 1)
 
         # update current hypervisor base port and base UDP
         base_udp = 0
@@ -812,13 +886,13 @@ class NETFile(object):
             self.add_connection(connection)
 
         self.apply_decorative_node_connections()
-        
+
         globals.GApp.mainWindow.treeWidget_TopologySummary.refresh()
         globals.GApp.dynagen.update_running_config()
         globals.GApp.workspace.projectFile = path
         globals.GApp.workspace.setWindowTitle("GNS3 - " + globals.GApp.workspace.projectFile)
         debug("Running config after importing: " + str(self.dynagen.running_config))
-        
+
         for node in globals.GApp.topology.nodes.itervalues():
             node.updateToolTips()
 
@@ -830,33 +904,31 @@ class NETFile(object):
             # Write out the config to a file
             file_path = os.path.normpath(globals.GApp.workspace.projectConfigs) + os.sep + device.name + '.cfg'
             if auto == False:
-                print unicode(translate("NETFile", "Exporting %s configuration to %s")) % (device.name, file_path)
+                print translate("NETFile", "Exporting %s configuration to %s") % (device.name, file_path)
         except lib.DynamipsError, msg:
             if auto == False:
-                print unicode(device.name + ': ' + translate("NETFile", "Dynamips error") + ': ') + unicode(msg)
+                print translate("NETFile", "%s: Dynamips error: %s") % (device.name, msg)
             return
         except lib.DynamipsWarning, msg:
             if auto == False:
-                print unicode(device.name + ': ' + translate("NETFile", "Dynamips warning") + ': ') + unicode(msg)
+                print translate("NETFile", "%s: Dynamips warning: %s") % (device.name, msg)
             return
         except:
             error('Unknown error exporting config for ' + device.name)
             return
         try:
-            f = open(file_path, 'w')
+            f = open(file_path, 'w') #export_router_config
             f.write(config)
             f.close()
             device.cnfg = file_path
             self.dynagen.running_config[device.dynamips.host + ':' + str(device.dynamips.port)]['ROUTER ' + device.name]['cnfg'] = file_path
         except IOError, e:
-            QtGui.QMessageBox.critical(globals.GApp.mainWindow, 
-                                      unicode(device.name) + ': ' + translate("NETFile", "IOError"), unicode(translate("NETFile", "%s: IO Error: %s")) % (file_path, unicode(e)))
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("NETFile", "%s: IOError") % device.name, translate("NETFile", "%s: IO Error: %s") % (file_path, e))
             return
 
     def export_net_file(self, path, auto=False):
         """ Export a .net file
         """
-
         # remove unused hypervisors
         hypervisors = self.dynagen.dynamips.copy()
         for (name, hypervisor) in hypervisors.iteritems():
@@ -873,6 +945,7 @@ class NETFile(object):
             hypervisor.configchange = True
         self.dynagen.defaults_config_ran = False
         self.dynagen.update_running_config()
+        debugmsg(3, ("NETFile.py: export_net_file() dynagen.running_config = ", self.dynagen.running_config))
         debug("Running config: " + str(self.dynagen.running_config))
 
         for device in self.dynagen.devices.values():
@@ -916,15 +989,15 @@ class NETFile(object):
                 if not self.dynagen.running_config.has_key('GNS3-DATA'):
                     self.dynagen.running_config['GNS3-DATA'] = {}
                 self.dynagen.running_config['GNS3-DATA']['NOTE ' + str(note_nb)] = {}
-                config = self.dynagen.running_config['GNS3-DATA']['NOTE ' + str(note_nb)] 
+                config = self.dynagen.running_config['GNS3-DATA']['NOTE ' + str(note_nb)]
 
-                config['text'] = unicode(item.toPlainText()).replace("\n", "\\n")
+                config['text'] = unicode(item.toPlainText(), 'utf-8', errors='replace').replace("\n", "\\n")
                 config['x'] = item.x()
                 config['y'] = item.y()
-                
+
                 if item.deviceName and item.deviceIf:
                     config['interface'] = item.deviceName + ' ' + item.deviceIf
-                    
+
                 if item.font() != QtGui.QFont("TypeWriter", 10, QtGui.QFont.Bold):
                     config['font'] = '"' + str(item.font().toString()) + '"'
                 if item.rotation != 0:
@@ -933,16 +1006,16 @@ class NETFile(object):
                     config['color'] = '"' + str(item.defaultTextColor().name()) + '"'
 
                 zvalue = item.zValue()
-                if zvalue != 1:
+                if zvalue != 2:
                     config['z'] = zvalue
                 note_nb += 1
-                
+
             # record shape items
             elif isinstance(item, AbstractShapeItem):
                 if not self.dynagen.running_config.has_key('GNS3-DATA'):
                     self.dynagen.running_config['GNS3-DATA'] = {}
                 self.dynagen.running_config['GNS3-DATA']['SHAPE ' + str(shape_nb)] = {}
-                config = self.dynagen.running_config['GNS3-DATA']['SHAPE ' + str(shape_nb)] 
+                config = self.dynagen.running_config['GNS3-DATA']['SHAPE ' + str(shape_nb)]
                 if isinstance(item, QtGui.QGraphicsRectItem):
                     config['type'] = 'rectangle'
                 else:
@@ -984,14 +1057,14 @@ class NETFile(object):
                 config['x'] = item.x()
                 config['y'] = item.y()
                 zvalue = item.zValue()
-                if zvalue > 0:
+                if zvalue != 0:
                     config['z'] = zvalue
                 pix_nb += 1
             elif isinstance(item, DecorativeNode):
                 if not self.dynagen.running_config.has_key('GNS3-DATA'):
                     self.dynagen.running_config['GNS3-DATA'] = {}
                 self.dynagen.running_config['GNS3-DATA']['NODE ' + item.hostname] = {}
-                config = self.dynagen.running_config['GNS3-DATA']['NODE ' + item.hostname] 
+                config = self.dynagen.running_config['GNS3-DATA']['NODE ' + item.hostname]
                 config['symbol'] = item.type
                 config['x'] = item.x()
                 config['y'] = item.y()
@@ -1012,11 +1085,11 @@ class NETFile(object):
                     item.showHostname()
                 # record node x & y positions
                 if not item.d:
-                    print item.hostname + unicode(' ' + translate("NETFile", "must be connected or have a hypervisor set in order to be registered"))
+                    print translate("NETFile", "%s must be connected or have a hypervisor set in order to be registered") % item.hostname
                     continue
                 if not item.default_symbol:
                     self.dynagen.running_config[item.d][item.get_running_config_name()]['symbol'] = item.type
-                try:          
+                try:
                     self.dynagen.running_config[item.d][item.get_running_config_name()]['x'] = item.x()
                     self.dynagen.running_config[item.d][item.get_running_config_name()]['y'] = item.y()
                     zvalue = item.zValue()
@@ -1065,18 +1138,25 @@ class NETFile(object):
                 autostart = True
                 break
         self.dynagen.running_config['autostart'] = autostart
-        
+
+        # add GNS3 version
+        from __main__ import VERSION
+        self.dynagen.running_config['version'] = VERSION
+
         if globals.GApp.systconf['general'].relative_paths:
             # Change absolute paths to relative paths if same base as the config file
             for hypervisor in self.dynagen.dynamips.values():
                 if isinstance(hypervisor, qlib.Qemu):
-                    h = 'qemu ' + hypervisor.host + ":" + str(hypervisor.port)          
+                    h = 'qemu ' + hypervisor.host + ":" + str(hypervisor.port)
+                elif isinstance(hypervisor, vboxlib.VBox):
+                    h = 'vbox ' + hypervisor.host + ":" + str(hypervisor.port)
                 else:
                     h = hypervisor.host + ":" + str(hypervisor.port)
                 config = self.dynagen.running_config[h]
+                #if config.has_key('workingdir') and not isinstance(hypervisor, vboxlib.VBox): # Dirty hack.
                 if config.has_key('workingdir'):
                     config['workingdir'] = self.convert_to_relpath(config['workingdir'], path)
-    
+
                 for model in dynagen_namespace.DEVICETUPLE:
                     if config.has_key(model):
                         # ASA has no image
@@ -1089,23 +1169,27 @@ class NETFile(object):
                             config[model]['image2'] = self.convert_to_relpath(config[model]['image2'], path)
                         else:
                             config[model]['image'] = self.convert_to_relpath(config[model]['image'], path)
-         
+
                 for subsection in config.sections:
                     device = config[subsection]
                     if device.has_key('cnfg') and device['cnfg']:
                         device['cnfg'] = self.convert_to_relpath(device['cnfg'], path)
-        
+
         self.dynagen.running_config.filename = path
         try:
-            self.dynagen.running_config.write()  
+            debugmsg(3, ("NETFile.py: writing... dynagen.running_config = ", self.dynagen.running_config))
+            self.dynagen.running_config.write()
         except IOError, e:
-            QtGui.QMessageBox.critical(globals.GApp.mainWindow, 
-                                      unicode(device.name) + ': ' + translate("NETFile", "IOError"), unicode(translate("NETFile", "%s: IO Error: %s")) % (path, unicode(e)))        
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("NETFile", "%s: IOError") % device.name, translate("NETFile", "%s: IO Error: %s") % (path, e))
         self.dynagen.running_config.filename = None
-        
+
     def convert_to_relpath(self, path, config_path):
         """ Returns a relative path when the config path and another path share a common base directory
         """
+        debugmsg(3, "NETFile.py: convert_to_relpath(%s, %s)" % (unicode(path), unicode(config_path)))
+        # Workaround, if remote hypervisor doesn't have workdir set:
+        if path == None:
+            return None
 
         real_image_path = os.path.realpath(path)
         config_dir = os.path.dirname(os.path.realpath(config_path))
