@@ -16,14 +16,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import re
 import GNS3.Globals as globals
 from PyQt4 import QtCore, QtGui, QtSvg
 from GNS3.Utils import translate, debug
-import GNS3.UndoFramework as undo 
+import GNS3.UndoFramework as undo
 
 class AbstractNode(QtSvg.QGraphicsSvgItem):
     """ AbstractNode class
@@ -44,7 +44,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
 
         self.dynagen = globals.GApp.dynagen
         self.default_symbol = True
-        
+
         # status used in the topology summary
         self.state = 'stopped'
 
@@ -63,14 +63,28 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         self.setFlags(flags)
         self.setAcceptsHoverEvents(True)
         self.setSharedRenderer(self.__render_normal)
-        
+
         # x&y position for hostname
         self.hostname_xpos = None
         self.hostname_ypos = None
-        
+
         self.default_hostname_xpos = None
         self.default_hostname_ypos = None
 
+        # Used by the undo process to prevent to ask again to choose an image
+        self.image_reference = None
+
+        self.setZValue(1)
+
+    def setRenderers(self, render_normal, render_select):
+        """ renderer_normal: QtSvg.QSvgRenderer
+            renderer_select: QtSvg.QSvgRenderer
+        """
+
+        self.__render_normal = render_normal
+        self.__render_select = render_select
+        self.setSharedRenderer(self.__render_normal)
+
     def getState(self):
         """ Returns the current node state
         """
@@ -80,12 +94,12 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
     def updateToolTips(self):
         """ Update node and child links tooltip
         """
-            
+
         self.setCustomToolTip()
         for edge in self.__edgeList:
             edge.setCustomToolTip()
         globals.GApp.mainWindow.treeWidget_TopologySummary.refresh()
-        
+
     def setUndoConfig(self, config, prevConfig):
         """ Set a new config to be put on the undo stack
         """
@@ -103,7 +117,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         if ok and text:
             text = unicode(text)
             if not re.search(r"""^[\w,.]*$""", text, re.UNICODE):
-                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hostname"), 
+                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hostname"),
                                            translate("AbstractNode", "Please use only alphanumeric characters"))
                 self.changeHostname()
                 return
@@ -115,16 +129,16 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
 
             command = undo.NewHostname(self, text)
             globals.GApp.topology.undoStack.push(command)
-            
+
     def changeHypervisor(self):
         """ Called to change an hypervisor
         """
-        
+
         if len(self.__edgeList) > 0:
-            QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("AbstractNode", "Hypervisor"), 
+            QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("AbstractNode", "Hypervisor"),
                                       translate("AbstractNode", "The device must have no connection to other devices in order to change its hypervisor"))
             return
-        
+
         if self.d:
             currentHypervisor = self.d
         else:
@@ -132,11 +146,11 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         (text, ok) = QtGui.QInputDialog.getText(globals.GApp.mainWindow, translate("AbstractNode", "Set hypervisor"),
                                     translate("AbstractNode", "New hypervisor:"), QtGui.QLineEdit.Normal,
                                     currentHypervisor)
-        
+
         if ok and text:
             hypervisor = unicode(text)
             if not re.search(r"""^.*:[0-9]*$""", text, re.UNICODE):
-                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hypervisor"), 
+                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hypervisor"),
                                            translate("AbstractNode", "Invalid format for hypervisor (hostname:port is required)"))
                 return
             (host, port) = hypervisor.rsplit(':',  1)
@@ -146,10 +160,17 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
                 dynamips_hypervisor = self.dynagen.dynamips[hypervisor]
             else:
                 debug("Connection to an hypervisor: " + hypervisor)
+
+                # use project workdir
+                if globals.GApp.workspace.projectWorkdir:
+                    self.dynagen.defaults_config['workingdir'] = globals.GApp.workspace.projectWorkdir
+                elif globals.GApp.systconf['dynamips'].workdir:
+                    self.dynagen.defaults_config['workingdir'] = globals.GApp.systconf['dynamips'].workdir
+
                 dynamips_hypervisor = self.dynagen.create_dynamips_hypervisor(host, int(port))
                 if not dynamips_hypervisor:
                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hypervisor"),
-                                               unicode(translate("AbstractNode", "Can't connect to the hypervisor on %s")) % hypervisor)
+                                               translate("AbstractNode", "Can't connect to the hypervisor on %s") % hypervisor)
                     if self.dynagen.dynamips.has_key(hypervisor):
                         del self.dynagen.dynamips[hypervisor]
                     return
@@ -160,10 +181,10 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
                 self.dynagen.running_config[host + ':' + port][self.get_running_config_name()] = config
                 del self.dynagen.running_config[self.d][self.get_running_config_name()]
             self.set_hypervisor(dynamips_hypervisor)
-            self.get_dynagen_device()
+            self.reconfigNode(self.hostname)
             self.dynagen.update_running_config()
             QtGui.QMessageBox.information(globals.GApp.mainWindow, translate("AbstractNode", "Hypervisor"),
-                                           unicode(translate("AbstractNode", "New hypervisor %s has been set on device %s")) % (hypervisor, self.hostname))
+                                          translate("AbstractNode", "New hypervisor %s has been set on device %s") % (hypervisor, self.hostname))
 
     def changeConsolePort(self):
         """ Called to change the console port
@@ -171,7 +192,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
 
         device = self.get_dynagen_device()
         (port, ok) = QtGui.QInputDialog.getInteger(globals.GApp.mainWindow, translate("AbstractNode", "Change the console port"),
-                                          unicode(translate("AbstractNode", "Console port for %s:")) % self.hostname, device.console, 1, 65535, 1)
+                                                   translate("AbstractNode", "Console port for %s:") % self.hostname, device.console, 1, 65535, 1)
         if ok and device.console != port:
 
             command = undo.NewConsolePort(self, port)
@@ -180,6 +201,27 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
                 QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Console port"), unicode(command.getStatus()))
                 globals.GApp.topology.undoStack.undo()
 
+    def changeAUXPort(self):
+        """ Called to change the aux port
+        """
+
+        device = self.get_dynagen_device()
+        if not device.aux:
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "AUX port"),
+                                       translate("AbstractNode", "AUX port not available for this router model or base AUX port is set to 0 in preferences"))
+            return False
+
+        current_aux = device.aux
+        (port, ok) = QtGui.QInputDialog.getInteger(globals.GApp.mainWindow, translate("AbstractNode", "Change the aux port"),
+                                                   translate("AbstractNode", "AUX port for %s:") % self.hostname, current_aux, 1, 65535, 1)
+        if ok and device.aux != port:
+
+            command = undo.NewAUXPort(self, port)
+            globals.GApp.topology.undoStack.push(command)
+            if command.getStatus() != None:
+                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "AUX port"), unicode(command.getStatus()))
+                globals.GApp.topology.undoStack.undo()
+
     def paint(self, painter, option, widget=None):
         """ Don't show the selection rectangle
         """
@@ -189,15 +231,15 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
             _local_option.state = QtGui.QStyle.State_None
 
         QtSvg.QGraphicsSvgItem.paint(self, painter, _local_option, widget)
-        
+
         # Don't draw if not activated
         if globals.GApp.workspace.flg_showLayerPos == False:
             return
-        
+
         # Show layer level of this node
         brect = self.boundingRect()
         center = self.mapFromItem(self, brect.width() / 2.0, brect.height() / 2.0)
-        
+
         painter.setBrush(QtCore.Qt.red)
         painter.setPen(QtCore.Qt.red)
         painter.drawRect((brect.width() / 2.0) - 10, (brect.height() / 2.0) - 10, 20,20)
@@ -229,10 +271,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         """
 
         # update tool tip
-        try:
-            self.updateToolTips()
-        except:
-            pass
+        self.setCustomToolTip()
         if not self.isSelected() and self.__render_select:
             self.setSharedRenderer(self.__render_select)
 
@@ -257,6 +296,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         """
 
         if edge in self.__edgeList:
+            edge.stopCapturing(showMessage=False)
             self.__edgeList.remove(edge)
 
     def getEdgeList(self):
@@ -269,7 +309,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         """ Set a custom tool tip
         """
 
-        self.setToolTip(unicode(translate("AbstractNode", "Hostname: %s")) % self.hostname)
+        self.setToolTip(translate("AbstractNode", "Hostname: %s") % self.hostname)
 
     def keyPressEvent(self, event):
         """ Key press handler
@@ -314,6 +354,16 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
             interface_list.add(interface)
         return interface_list
 
+    def getConnectedLinkByName(self, ifname):
+        """ Returns the link object corresponding to an interface name
+        """
+
+        for edge in self.__edgeList:
+            interface = edge.getLocalInterface(self)
+            if interface == ifname:
+                return edge
+        return None
+
     def getConnectedNeighbor(self, ifname):
         """ Returns the connected neighbor's node and interface
         """
@@ -322,21 +372,26 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
             interface = edge.getLocalInterface(self)
             if interface == ifname:
                 return edge.getConnectedNeighbor(self)
+        return None
 
-    def showMenuInterface(self):
+    def showMenuInterface(self, unavailable_interfaces=[]):
         """ Show a contextual menu to choose an interface on a specific node
-            node: node instance
         """
 
         globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
         menu = QtGui.QMenu()
+        self.connect(menu, QtCore.SIGNAL("hovered(QAction *)"), self._actionHovered)
         interfaces_list = self.getInterfaces()
         if len(interfaces_list) == 0:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Connection"), translate("AbstractNode", "No interface available, please configure this device"))
             return
         connected_list = self.getConnectedInterfaceList()
         for interface in interfaces_list:
-            if interface in connected_list:
+            if interface in unavailable_interfaces:
+                # interface cannot be chosen by user (grayed out)
+                action = menu.addAction(QtGui.QIcon(':/icons/led_green.svg'), interface)
+                action.setDisabled(True)
+            elif interface in connected_list:
                 # already connected interface
                 menu.addAction(QtGui.QIcon(':/icons/led_green.svg'), interface)
             else:
@@ -348,6 +403,10 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         menu.exec_(QtGui.QCursor.pos())
         globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
 
+    # Overloaded in Cloud
+    def _actionHovered(self, action):
+        pass
+
     def showHostname(self):
         """ Show the hostname on the scene
         """
@@ -363,14 +422,12 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         self.textItem.setFlag(self.textItem.ItemIsMovable)
         if self.hostname_xpos == None and self.hostname_ypos == None:
             # use default positions
-            if self.default_hostname_xpos == None and self.default_hostname_ypos == None:
-                # compute default positions once
-                textrect = self.textItem.boundingRect()
-                textmiddle = textrect.topRight() / 2
-                noderect = self.boundingRect()
-                nodemiddle = noderect.topRight() / 2
-                self.default_hostname_xpos = nodemiddle.x() - textmiddle.x()
-                self.default_hostname_ypos = -25
+            textrect = self.textItem.boundingRect()
+            textmiddle = textrect.topRight() / 2
+            noderect = self.boundingRect()
+            nodemiddle = noderect.topRight() / 2
+            self.default_hostname_xpos = nodemiddle.x() - textmiddle.x()
+            self.default_hostname_ypos = -25
             self.textItem.setPos(self.default_hostname_xpos, self.default_hostname_ypos)
         else:
             # use user defined positions
