@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,19 +16,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# http://www.gns3.net/contact
 #
 
 from GNS3.Node.AbstractNode import AbstractNode
 from PyQt4 import QtCore, QtGui
 from GNS3.Utils import translate, debug
 import GNS3.Dynagen.dynamips_lib as lib
-import GNS3.Dynagen.dynagen as dynagen_namespace
 import GNS3.Globals as globals
 
-ethsw_id = 0
+ethsw_id = 1
 
-def init_ethsw_id(id = 0):
+def init_ethsw_id(id = 1):
     global ethsw_id
     ethsw_id = id
 
@@ -42,13 +41,13 @@ class ETHSW(AbstractNode):
 
         # assign a new hostname
         global ethsw_id
-        
+
         # check if hostname has already been assigned
         for node in globals.GApp.topology.nodes.itervalues():
             if 'SW' + str(ethsw_id) == node.hostname:
                 ethsw_id = ethsw_id + 1
                 break
-        
+
         self.hostname = 'SW' + str(ethsw_id)
         ethsw_id = ethsw_id + 1
         AbstractNode.setCustomToolTip(self)
@@ -88,7 +87,7 @@ class ETHSW(AbstractNode):
         self.hostname = hostname
         self.e = 'ETHSW ' + self.hostname
         self.updateToolTips()
-        
+
     def setCustomToolTip(self):
         """ Set a custom tool tip
         """
@@ -117,6 +116,15 @@ class ETHSW(AbstractNode):
             self.config['ports'][port] = 'access'
             self.config['vlans'][1].append(port)
 
+    def duplicate_config(self):
+        """ Returns a copy of the configuration
+        """
+
+        config = self.config.copy()
+        config['vlans'] = self.config['vlans'].copy()
+        config['ports'] = self.config['ports'].copy()
+        return (config)
+
     def get_config(self):
         """ Returns the local configuration copy
         """
@@ -128,10 +136,13 @@ class ETHSW(AbstractNode):
             config: dict
         """
 
-        self.config = config
+        self.config = config.copy()
+        self.config['vlans'] = config['vlans'].copy()
+        self.config['ports'] = config['ports'].copy()
         globals.GApp.topology.changed = True
+        self.mapping()
 
-    def set_hypervisor(self,  hypervisor):
+    def set_hypervisor(self, hypervisor):
         """ Records a hypervisor
             hypervisor: object
         """
@@ -139,10 +150,21 @@ class ETHSW(AbstractNode):
         self.hypervisor = hypervisor
         self.d = self.hypervisor.host + ':' + str(self.hypervisor.port)
 
+    def autoAllocateFreePort(self):
+        """ Auto allocate one additional free port when all ports are occupied
+        """
+
+        if len(self.config['ports']) == len(self.getConnectedInterfaceList()):
+            port = max(self.config['ports']) + 1
+            self.config['ports'][port] = 'access'
+            self.config['vlans'][1].append(port)
+
+
     def getInterfaces(self):
         """ Returns all interfaces
         """
 
+        self.autoAllocateFreePort()
         ports = map(str, self.config['ports'].keys())
         return (ports)
 
@@ -185,8 +207,8 @@ class ETHSW(AbstractNode):
         self.create_config()
         return True
 
-    def startNode(self):
-        """ Start the node
+    def mapping(self):
+        """ Configure Ethernet port mapping
         """
 
         connected_interfaces = map(int, self.getConnectedInterfaceList())
@@ -196,13 +218,29 @@ class ETHSW(AbstractNode):
                     if not self.ethsw.mapping.has_key(port):
                         (destnode, destinterface)= self.getConnectedNeighbor(str(port))
                         porttype = self.config['ports'][port]
+                        if self.ethsw.dynamips.intversion < 208.3 and porttype == 'qinq':
+                            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("ETHSW", "Port type"),  translate("ETHSW", "QinQ is only supported with Dynamips > 0.2.8 RC2"))
+                            return
+
                         if destinterface.lower()[:3] == 'nio':
                             debug("ethsw_map: " + str(port) + ' to ' + porttype + ' ' + str(vlan) + ' ' + destinterface)
                             self.dynagen.ethsw_map(self.ethsw, port, porttype + ' ' + str(vlan) + ' ' + destinterface)
                         else:
                             debug("ethsw_map: " + str(port) + ' to ' + porttype + ' ' + str(vlan))
                             self.dynagen.ethsw_map(self.ethsw, port, porttype + ' ' + str(vlan))
+                    elif self.ethsw.mapping.has_key(port):
+                        porttype = self.config['ports'][port]
+                        # check if the vlan or port type has changed
+                        # WARNING: see unset_port()
+                        if (vlan != self.ethsw.mapping[port][1]) or (porttype != self.ethsw.mapping[port][0]):
+                            self.ethsw.unset_port(port)
+                            self.dynagen.ethsw_map(self.ethsw, port, porttype + ' ' + str(vlan))
 
+    def startNode(self):
+        """ Start the node
+        """
+
+        self.mapping()
         self.startupInterfaces()
         self.state = 'running'
         globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, 'running')
@@ -214,6 +252,7 @@ class ETHSW(AbstractNode):
         """
 
         if globals.addingLinkFlag and globals.currentLinkType != globals.Enum.LinkType.Manual and event.button() == QtCore.Qt.LeftButton:
+            self.autoAllocateFreePort()
             connected_ports = self.getConnectedInterfaceList()
             for port in self.config['ports'].keys():
                 if not str(port) in connected_ports:
