@@ -16,10 +16,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
-import os, shutil, glob, sys, base64
+import os, shutil, glob, sys, base64, time
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Dynagen.dynagen as dynagen_namespace
@@ -120,7 +120,7 @@ class IOSRouter(AbstractNode):
             'mac',
             'iomem',
             'npe',
-            'midplane', 
+            'midplane',
             ]
 
     def __del__(self):
@@ -152,12 +152,25 @@ class IOSRouter(AbstractNode):
         self.r = 'ROUTER ' + self.hostname
         self.updateToolTips()
 
+    def changeHostname(self):
+        """ Called to change the hostname
+        """
+
+        if self.router.state != 'stopped':
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("IOSRouter", "New hostname"),
+                                       translate("IOSRouter", "Cannot change the hostname of a running device"))
+            return
+        AbstractNode.changeHostname(self)
+
     def setCustomToolTip(self):
         """ Set a custom tool tip
         """
 
         if self.router:
-            self.setToolTip(self.router.info())
+            try:
+                self.setToolTip(self.router.info())
+            except:
+                AbstractNode.setCustomToolTip(self)
         else:
             AbstractNode.setCustomToolTip(self)
 
@@ -215,7 +228,7 @@ class IOSRouter(AbstractNode):
         config = self.local_config.copy()
         config['slots'] = list(self.local_config['slots'])
         return config
-        
+
     def set_config(self, config):
         """ Set a configuration in Dynamips
             config: dict
@@ -240,12 +253,12 @@ class IOSRouter(AbstractNode):
         for module_name in config['slots']:
             if module_name and self.router.slot[slot_number] == None:
                 self.set_slot(slot_number, module_name)
-                if not self.router.slot[slot_number].can_be_removed():
+                if self.router.slot[slot_number] and not self.router.slot[slot_number].can_be_removed():
                     slot_changed = True
             elif self.router.slot[slot_number] and module_name and module_name != self.router.slot[slot_number].adapter:
                 self.clean_slot(self.router.slot[slot_number])
                 self.set_slot(slot_number, module_name)
-                if not self.router.slot[slot_number].can_be_removed():
+                if self.router.slot[slot_number] and not self.router.slot[slot_number].can_be_removed():
                     slot_changed = True
             elif module_name == None and self.router.slot[slot_number]:
                 self.clean_slot(self.router.slot[slot_number])
@@ -254,15 +267,24 @@ class IOSRouter(AbstractNode):
         if slot_changed and self.router.model != 'c7200' and self.state == 'running':
             QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("IOSRouter", "Slots"), translate("IOSRouter", "You have to restart this router to use new modules"))
 
-        # configure wics if available
-        if config['wics']:
-            wic_number = 0
-            for wic_name in config['wics']:
-                if wic_name and not self.router.slot[0].wics[wic_number]:
-                    # consider that all wics are in slot 0
-                    debug('Install ' + wic_name + ' in wic port ' + str(wic_number))
-                    self.router.installwic(wic_name, 0, wic_number)
-                wic_number += 1
+        try:
+            # configure wics if available
+            if config['wics']:
+                wic_number = 0
+                for wic_name in config['wics']:
+                    if wic_name and not self.router.slot[0].wics[wic_number]:
+                        # consider that all wics are in slot 0
+                        debug('Install ' + wic_name + ' in wic port ' + str(wic_number))
+                        self.router.installwic(wic_name, 0, wic_number)
+                    elif wic_name and self.router.slot[0].wics[wic_number] != wic_name:
+                        self.router.uninstallwic(wic_number)
+                        debug('Re-Install ' + wic_name + ' in wic port ' + str(wic_number))
+                        self.router.installwic(wic_name, 0, wic_number)
+                    elif not wic_name and self.router.slot[0].wics[wic_number]:
+                        self.router.uninstallwic(wic_number)
+                    wic_number += 1
+        except lib.DynamipsError, msg:
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("IOSRouter", "Dynamips error"),  unicode(msg))
 
         self.local_config = config.copy()
         self.local_config['slots'] = list(config['slots'])
@@ -305,9 +327,9 @@ class IOSRouter(AbstractNode):
     def changeStartupConfig(self):
         """ Called to change the startup-config
         """
-     
+
         startupConfigDlg = StartupConfigDialog(self.router)
-        startupConfigDlg.setWindowTitle(unicode(translate("IOSRouter", "Startup-Config for %s")) % self.hostname)
+        startupConfigDlg.setWindowTitle(translate("IOSRouter", "Startup-Config for %s") % self.hostname)
         startupConfigDlg.show()
         startupConfigDlg.exec_()
 
@@ -514,7 +536,7 @@ class IOSRouter(AbstractNode):
         self.set_config(self.local_config)
         for link in links:
             globals.GApp.topology.addLink(link.source.id, link.srcIf, link.dest.id, link.destIf)
-            
+
         # Update base config
         try:
             config = base64.decodestring(self.router.config_b64)
@@ -552,9 +574,12 @@ class IOSRouter(AbstractNode):
             module: object
         """
 
-        if module.can_be_removed():
-            module.remove()
-            self.router.slot[module.slot] = None
+        try:
+            if module.can_be_removed():
+                module.remove()
+                self.router.slot[module.slot] = None
+        except lib.DynamipsError, msg:
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("IOSRouter", "Dynamips error"),  unicode(msg))
 
     def getInterfaces(self):
         """ Returns all the router interfaces
@@ -606,14 +631,15 @@ class IOSRouter(AbstractNode):
             self.state = self.router.state
             self.updateToolTips()
             globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.router.state)
-            
+
     def reloadNode(self, progress=False):
         """ Reload this node
         """
- 
+
         if self.router.state != 'running':
             return
         self.stopNode(progress)
+        time.sleep(1)
         self.startNode(progress)
 
     def suspendNode(self, progress=False):
@@ -636,8 +662,19 @@ class IOSRouter(AbstractNode):
         """
 
         if self.router and self.router.state == 'running' and self.router.console:
-            console.connect(self.hypervisor.host,  self.router.console,  self.hostname)
-    
+            console.connect(self.hypervisor.host, self.router.console, self.hostname)
+
+    def aux(self):
+        """ Start a telnet console and connect it to this router's AUX port
+        """
+        if not self.router.aux:
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "AUX port"),
+                                       translate("AbstractNode", "AUX port not available for this router model or base AUX port is set to 0 in preferences"))
+            return False
+
+        if self.router and self.router.state == 'running':
+            console.connect(self.hypervisor.host, self.router.aux, self.hostname)
+
     def isStarted(self):
         """ Returns True if this router is started
         """
