@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,11 +16,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# http://www.gns3.net/contact
 #
 
+
 import GNS3.Globals as globals
-import GNS3.Ui.svg_resources_rc
 from PyQt4 import QtCore, QtGui
 from GNS3.Utils import translate
 from GNS3.Ui.Form_NodeConfigurator import Ui_NodeConfigurator
@@ -31,14 +31,13 @@ from GNS3.Node.IOSRouter3600 import IOSRouter3600
 from GNS3.Node.IOSRouter3700 import IOSRouter3700
 from GNS3.Node.IOSRouter7200 import IOSRouter7200
 from GNS3.Node.DecorativeNode import DecorativeNode
-from GNS3.Node.SIMHOST import SIMHOST
-from GNS3.Node.FW import FW
+from GNS3.Node.AnyEmuDevice import PIX, ASA, JunOS, IDS, QemuDevice
+from GNS3.Node.AnyVBoxEmuDevice import VBoxDevice
 from GNS3.Node.FRSW import FRSW
 from GNS3.Node.ETHSW import ETHSW
 from GNS3.Node.ATMSW import ATMSW
 from GNS3.Node.ATMBR import ATMBR
 from GNS3.Node.Cloud import Cloud
-from GNS3.Node.Hub import Hub
 
 class ConfigurationPageItem(QtGui.QTreeWidgetItem):
     """ Class implementing a QTreeWidgetItem holding the configuration page data.
@@ -101,7 +100,7 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
             # key : [display string, pixmap name, dialog module name, parent key,
             #        reference to configuration page (must always be last)]
             # The dialog module must have the module function create to create
-            # the configuration page. This must have the method save to save 
+            # the configuration page. This must have the method save to save
             # the settings.
             "Routers (1700)" : \
                 [translate("NodeConfigurator", "Routers c1700"), ":/symbols/router.normal.svg",
@@ -124,12 +123,24 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
             "Decorative Nodes":
                 [translate("NodeConfigurator", "Nodes"), ":/icons/node_conception.svg",
                  "Page_DecorativeNode", None, None],
-            "FW":
+            "PIX":
                 [translate("NodeConfigurator", "PIX firewalls"), ":/symbols/PIX_firewall.normal.svg",
-                 "Page_FW", None, None],
-            "HOST":
-                [translate("NodeConfigurator", "Hosts"), ":/symbols/computer.normal.svg",
-                 "Page_SIMHOST", None, None],
+                 "Page_PIX", None, None],
+            "ASA":
+                 [translate("NodeConfigurator", "ASA firewalls"), ":/symbols/PIX_firewall.normal.svg",
+                  "Page_ASA", None, None],
+            "JunOS":
+                  [translate("NodeConfigurator", "Juniper routers"), ":/symbols/router.normal.svg",
+                   "Page_JunOS", None, None],
+            "IDS":
+                  [translate("NodeConfigurator", "Cisco IDS"), ":/symbols/ids.normal.svg",
+                   "Page_IDS", None, None],
+            "Qemu":
+                [translate("NodeConfigurator", "Qemu guests"), ":/symbols/computer.normal.svg",
+                 "Page_Qemu", None, None],
+            "VBox":
+                [translate("NodeConfigurator", "VirtualBox guests"), ":/symbols/computer.normal.svg",
+                 "Page_VirtualBox", None, None],
             "FRSW":
                 [translate("NodeConfigurator", "Frame Relay switches"), ":/symbols/frame_relay_switch.normal.svg",
                  "Page_FRSW", None, None],
@@ -144,11 +155,8 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
                  "Page_ATMBR", None, None],
             "Clouds":
                 [translate("NodeConfigurator", "Clouds"), ":/symbols/cloud.normal.svg",
-                 "Page_Cloud", None, None],
-            "Hubs":
-                [translate("NodeConfigurator", "Hubs"), ":/symbols/hub.normal.svg",
-                 "Page_Hub", None, None]
-                 }
+                 "Page_Cloud", None, None]
+                }
 
         self.assocPage = {
                                      IOSRouter1700: "Routers (1700)",
@@ -158,14 +166,17 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
                                      IOSRouter3700: "Routers (3700)",
                                      IOSRouter7200: "Routers (7200)",
                                      DecorativeNode: "Decorative Nodes",
-                                     FW: "FW",
-                                     SIMHOST: "HOST",
+                                     PIX: "PIX",
+                                     ASA: "ASA",
+                                     JunOS: "JunOS",
+                                     IDS: "IDS",
+                                     QemuDevice: "Qemu",
+                                     VBoxDevice: "VBox",
                                      FRSW: "FRSW",
                                      ETHSW: "ETHSW",
                                      ATMSW: "ATMSW",
                                      ATMBR: "ATMBR",
                                      Cloud: "Clouds",
-                                     Hub: "Hubs"
                                     }
         self.__loadNodeItems()
 
@@ -192,7 +203,7 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
                 item.setExpanded(True)
                 self.itmDict[parent] = item
 
-        # create the children items 
+        # create the children items
         for node in self.nodeitems:
             if not self.assocPage.has_key(type(node)):
                 continue
@@ -228,17 +239,16 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
                 count = count - 1
             if not item.parent():
                 if last_item.parent():
-                    newLabel = unicode(translate("NodeConfigurator", "%s node")) % (unicode(last_item.text(0)))
+                    newLabel = translate("NodeConfigurator", "%s node") % (unicode(last_item.text(0)))
                     self.titleLabel.setText(newLabel)
-                newLabel = unicode(translate("NodeConfigurator", "%s group")) % (unicode(last_item.text(0)))
+                newLabel = translate("NodeConfigurator", "%s group") % (unicode(last_item.text(0)))
                 self.titleLabel.setText(newLabel)
                 return
 
         if count > 1:
-            pageTitle = unicode(translate("NodeConfigurator", "Group of %d %s")) \
-                % (count, unicode(last_item.parent().text(0)))
+            pageTitle = translate("NodeConfigurator", "Group of %d %s") % (count, unicode(last_item.parent().text(0)))
         else:
-            pageTitle = unicode(translate("NodeConfigurator", "%s node")) % (unicode(last_item.text(0)) )
+            pageTitle = translate("NodeConfigurator", "%s node") % (unicode(last_item.text(0)) )
         self.titleLabel.setText(pageTitle)
 
     def __importConfigurationPage(self, name):
@@ -345,10 +355,10 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
             self.on_cancelButton_clicked()
             QtGui.QDialog.reject(self)
         else:
-            self.on_applyButton_clicked()
+            self.on_applyButton_clicked(close=True)
             QtGui.QDialog.accept(self)
 
-    def on_applyButton_clicked(self):
+    def on_applyButton_clicked(self, close=False):
         """ Private slot called to apply the settings
         """
 
@@ -359,13 +369,19 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
                 if item.parent():
                         config = page.saveConfig(item.getIDs()[0])
                         node = globals.GApp.topology.getNode(item.getIDs()[0])
-                        node.set_config(config)
+                        if close:
+                            node.setUndoConfig(config, node.duplicate_config())
+                        else:
+                            node.set_config(config)
                 else:
                     children = item.getIDs()
                     for child in children:
                         config = page.saveConfig(child)
                         node = globals.GApp.topology.getNode(child)
-                        node.set_config(config)
+                        if close:
+                            node.setUndoConfig(config, node.duplicate_config())
+                        else:
+                            node.set_config(config)
 
     def on_cancelButton_clicked(self):
         """ Private slot called to cancel the settings
@@ -388,4 +404,4 @@ class NodeConfigurator(QtGui.QDialog, Ui_NodeConfigurator):
             for item in self.treeViewNodes.selectedItems():
                 node = globals.GApp.topology.getNode(item.getIDs()[0])
                 item.tmpConfig = node.get_config()
-                page.loadConfig(item.getIDs()[0],  item.tmpConfig)
+                page.loadConfig(item.getIDs()[0], item.tmpConfig)
