@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 from PyQt4 import QtGui, QtCore
@@ -32,10 +32,10 @@ class AbstractShapeItem(object):
         self.setAcceptsHoverEvents(True)
         self.border = 5
         self.rotation = 0
-        self.setZValue(-2)
-        
+        #self.setZValue(-2)
+
     def keyPressEvent(self, event):
-    
+
         key = event.key()
         if key == QtCore.Qt.Key_Left and event.modifiers() == QtCore.Qt.AltModifier and self.rotation > -360:
             if self.rotation:
@@ -51,7 +51,7 @@ class AbstractShapeItem(object):
             QtGui.QGraphicsItem.keyPressEvent(self, event)
 
     def mousePressEvent(self, event):
-        
+
         self.update()
         if event.pos().x() > (self.rect().right() - self.border) :
             self.setFlag(QtGui.QGraphicsItem.ItemIsMovable, False)
@@ -70,16 +70,16 @@ class AbstractShapeItem(object):
             self.edge = 'bottom'
 
         QtGui.QGraphicsItem.mousePressEvent(self, event)
-        
+
     def mouseReleaseEvent(self, event):
-    
+
         self.update()
         self.setFlag(QtGui.QGraphicsItem.ItemIsMovable)
         self.edge = None
         QtGui.QGraphicsItem.mouseReleaseEvent(self, event)
-        
+
     def mouseMoveEvent(self, event):
-        
+
         self.update()
         if hasattr(self, 'edge') and self.edge:
             r = self.rect()
@@ -123,7 +123,7 @@ class AbstractShapeItem(object):
                     self.edge = 'left'
 
         QtGui.QGraphicsItem.mouseMoveEvent(self, event)
-        
+
     def hoverMoveEvent(self, event):
 
 
@@ -143,9 +143,9 @@ class AbstractShapeItem(object):
             globals.GApp.scene.setCursor(QtCore.Qt.SizeAllCursor)
 
     def hoverLeaveEvent(self, event):
-    
+
         globals.GApp.scene.setCursor(QtCore.Qt.ArrowCursor)
-        
+
 
     def drawLayerInfo(self, painter):
 
@@ -155,13 +155,13 @@ class AbstractShapeItem(object):
 
         # Show layer level of this node
         brect = self.boundingRect()
-        
+
         # Don't draw if the object is too small ...
         if brect.width() < 20 or brect.height() < 20:
             return
-        
+
         center = self.mapFromItem(self, brect.width() / 2.0, brect.height() / 2.0)
-        
+
         painter.setBrush(QtCore.Qt.red)
         painter.setPen(QtCore.Qt.red)
         painter.drawRect((brect.width() / 2.0) - 10, (brect.height() / 2.0) - 10, 20,20)
@@ -169,11 +169,11 @@ class AbstractShapeItem(object):
         painter.setFont(QtGui.QFont("TypeWriter", 14, QtGui.QFont.Bold))
         zval = str(int(self.zValue()))
         painter.drawText(QtCore.QPointF(center.x() - 4, center.y() + 4), zval)
-            
+
 class Rectangle(AbstractShapeItem, QtGui.QGraphicsRectItem):
     """ Class to draw a rectangle on the scene
     """
-    
+
     def __init__(self, pos, size):
 
         QtGui.QGraphicsRectItem.__init__(self, 0, 0, size.width(), size.height())
@@ -181,16 +181,16 @@ class Rectangle(AbstractShapeItem, QtGui.QGraphicsRectItem):
         self.setPos(pos)
         pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
         self.setPen(pen)
-        
+
     def paint(self, painter, option, widget=None):
-        
+
         QtGui.QGraphicsRectItem.paint(self, painter, option, widget)
         self.drawLayerInfo(painter)
-        
+
 class Ellipse(AbstractShapeItem, QtGui.QGraphicsEllipseItem):
     """ Class to draw an ellipse on the scene
     """
-    
+
     def __init__(self, pos, size):
 
         QtGui.QGraphicsEllipseItem.__init__(self, 0, 0, size.width(), size.height())
@@ -198,7 +198,7 @@ class Ellipse(AbstractShapeItem, QtGui.QGraphicsEllipseItem):
         self.setPos(pos)
         pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.DashLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
         self.setPen(pen)
-        
+
     def paint(self, painter, option, widget=None):
 
         QtGui.QGraphicsEllipseItem.paint(self, painter, option, widget)
