@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import re, sys
@@ -34,7 +34,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         QtGui.QWidget.__init__(self)
         self.setupUi(self)
         self.setObjectName("Cloud")
-        
+            
         # connect slots
         self.connect(self.pushButtonAddGenericEth, QtCore.SIGNAL('clicked()'), self.slotAddGenEth)
         self.connect(self.pushButtonDeleteGenericEth, QtCore.SIGNAL('clicked()'), self.slotDeleteGenEth)
@@ -65,6 +65,8 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
         self.connect(self.listWidgetNull,  QtCore.SIGNAL('currentRowChanged(int)'),  self.slotNullselected)
         
         self.nios = []
+        self.rpcap_mapping = {}
+
         if sys.platform.startswith('win'):
             interfaces = getWindowsInterfaces()
         else:
@@ -100,17 +102,20 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
     def slotAddGenEth(self):
         """ Add a new generic Ethernet NIO
         """
-    
-        interface = unicode(self.lineEditGenEth.text())
+
+        interface = unicode(self.lineEditGenEth.text(), 'utf-8', errors='replace')
         if interface:
             if sys.platform.startswith('win'):
-                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*}).*""", interface)
+                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*})(.*)""", interface)
                 interface = match.group(1)
             nio = 'nio_gen_eth:' + interface.lower()
             if not nio in self.nios:
                 self.listWidgetGenericEth.addItem(nio)
                 self.nios.append(nio)
-        
+                if sys.platform.startswith('win'):
+                    name_match = re.search(r"""^\ :\ (.*)\ on local host:.*""", match.group(2))
+                    self.rpcap_mapping[nio] = name_match.group(1)
+
     def slotDeleteGenEth(self):
         """ Delete the selected generic Ethernet NIO
         """
@@ -120,7 +125,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             nio = unicode(item.text())
             connected_ports = self.node.getConnectedInterfaceList()
             if nio in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', translate("Page_Cloud", "A link is connected with NIO %s") % nio)
                 return
             self.nios.remove(nio)
             self.listWidgetGenericEth.takeItem(self.listWidgetGenericEth.currentRow())
@@ -154,7 +159,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             nio = unicode(item.text())
             connected_ports = self.node.getConnectedInterfaceList()
             if nio in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', translate("Page_Cloud", "A link is connected with NIO %s") % nio)
                 return
             self.nios.remove(nio)
             self.listWidgetLinuxEth.takeItem(self.listWidgetLinuxEth.currentRow())
@@ -183,7 +188,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             nio = unicode(item.text())
             connected_ports = self.node.getConnectedInterfaceList()
             if nio in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', translate("Page_Cloud", "A link is connected with NIO %s") % nio)
                 return
             self.nios.remove(nio)
             self.listWidgetUDP.takeItem(self.listWidgetUDP.currentRow())
@@ -231,7 +236,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             nio = unicode(item.text())
             connected_ports = self.node.getConnectedInterfaceList()
             if nio in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', translate("Page_Cloud", "A link is connected with NIO %s") % nio)
                 return
             self.nios.remove(nio)
             self.listWidgetTAP.takeItem(self.listWidgetTAP.currentRow())
@@ -267,7 +272,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             nio = unicode(item.text())
             connected_ports = self.node.getConnectedInterfaceList()
             if nio in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', translate("Page_Cloud", "A link is connected with NIO %s") % nio)
                 return
             self.nios.remove(nio)
             self.listWidgetUNIX.takeItem(self.listWidgetUNIX.currentRow())
@@ -315,7 +320,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             nio = unicode(item.text())
             connected_ports = self.node.getConnectedInterfaceList()
             if nio in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', translate("Page_Cloud", "A link is connected with NIO %s") % nio)
                 return
             self.nios.remove(nio)
             self.listWidgetVDE.takeItem(self.listWidgetVDE.currentRow())
@@ -362,7 +367,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             nio = unicode(item.text())  
             connected_ports = self.node.getConnectedInterfaceList()
             if nio in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', unicode(translate("Page_Cloud", "A link is connected with NIO %s")) % nio)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'NIOs', translate("Page_Cloud", "A link is connected with NIO %s") % nio)
                 return
             self.nios.remove(nio)
             self.listWidgetNull.takeItem(self.listWidgetNull.currentRow())
@@ -435,6 +440,7 @@ class Page_Cloud(QtGui.QWidget, Ui_CloudPage):
             Cloudconfig  = self.node.duplicate_config()
 
         Cloudconfig['nios'] = self.nios
+        Cloudconfig['rpcap_mapping'] = self.rpcap_mapping
 
         return Cloudconfig
             
