@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import GNS3.Globals as globals
@@ -79,22 +79,32 @@ class Page_ETHSW(QtGui.QWidget, Ui_ETHSWPage):
         type = str(self.comboBoxPortType.currentText())
         
         if self.ports.has_key(port):
-            QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, translate("Page_ETHSW",  "Add port"),  translate("Page_ETHSW",  "Port already exists"))
-            return
+            # update vlan for a given port
 
-        item = QtGui.QTreeWidgetItem(self.treeWidgetPorts)
-        item.setText(0, str(port))
-        item.setText(1, str(vlan))
-        item.setText(2, type)
-        self.treeWidgetPorts.addTopLevelItem(item)
-        
-        self.spinBoxPort.setValue(port + 1)
+            item = self.treeWidgetPorts.findItems(str(port), QtCore.Qt.MatchFixedString)[0]
+            previous_vlan = int(item.text(1))
+            item.setText(1, str(vlan))
+            item.setText(2, type)
+
+            self.vlans[previous_vlan].remove(port)
+            if len(self.vlans[previous_vlan]) == 0:
+                del self.vlans[previous_vlan]
+
+        else:
+
+            item = QtGui.QTreeWidgetItem(self.treeWidgetPorts)
+            item.setText(0, str(port))
+            item.setText(1, str(vlan))
+            item.setText(2, type)
+            self.treeWidgetPorts.addTopLevelItem(item)
+            
         self.ports[port] = type
         if not self.vlans.has_key(vlan):
             self.vlans[vlan] = []
         if not port in self.vlans[vlan]:
             self.vlans[vlan].append(port)
-        
+            
+        self.spinBoxPort.setValue(max(self.ports) + 1)
         self.treeWidgetPorts.resizeColumnToContents(0)
         
     def slotDeletePort(self):
@@ -107,13 +117,15 @@ class Page_ETHSW(QtGui.QWidget, Ui_ETHSWPage):
             vlan = int(item.text(1))
             connected_ports = self.node.getConnectedInterfaceList()
             if str(port) in connected_ports:
-                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'Ports', unicode(translate("Page_ETHSW", "A link is connected in port %i")) % port)
+                QtGui.QMessageBox.critical(globals.nodeConfiguratorWindow, 'Ports', translate("Page_ETHSW", "A link is connected in port %i") % port)
                 return
             del self.ports[port]
             self.vlans[vlan].remove(port)
             if len(self.vlans[vlan]) == 0:
                 del self.vlans[vlan]
             self.treeWidgetPorts.takeTopLevelItem(self.treeWidgetPorts.indexOfTopLevelItem(item))
+
+        self.spinBoxPort.setValue(max(self.ports) + 1)
         
     def loadConfig(self, id, config = None):
         """ Load the config
@@ -143,6 +155,7 @@ class Page_ETHSW(QtGui.QWidget, Ui_ETHSWPage):
                     self.vlans[vlan].append(port)
         self.treeWidgetPorts.resizeColumnToContents(0)
         self.treeWidgetPorts.resizeColumnToContents(1)
+        self.spinBoxPort.setValue(max(self.ports) + 1)
 
     def saveConfig(self, id, config = None):
         """ Save the config
