@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import sys, os
@@ -25,6 +25,7 @@ from PyQt4 import QtGui, QtCore
 from GNS3.Ui.ConfigurationPages.Form_PreferencesCapture import Ui_PreferencesCapture
 from GNS3.Config.Objects import systemCaptureConf
 from GNS3.Utils import fileBrowser, translate
+from GNS3.Config.Defaults import CAPTURE_PRESET_CMDS, CAPTURE_DEFAULT_CMD, CAPTURE_DEFAULT_WORKDIR
 from GNS3.Config.Config import ConfDB
 
 class UiConfig_PreferencesCapture(QtGui.QWidget, Ui_PreferencesCapture):
@@ -34,8 +35,20 @@ class UiConfig_PreferencesCapture(QtGui.QWidget, Ui_PreferencesCapture):
         QtGui.QWidget.__init__(self)
         Ui_PreferencesCapture.setupUi(self, self)
         self.connect(self.CaptureWorkingDirectory_Browser, QtCore.SIGNAL('clicked()'), self.__setCaptureWorkdir)
+        self.connect(self.pushButtonUsePresets, QtCore.SIGNAL('clicked()'), self.__setPresetsCmd)
+        
+        # TODO: implement named pipe method: http://wiki.wireshark.org/CaptureSetup/Pipes#Named_pipes
+        for (name, cmd) in sorted(CAPTURE_PRESET_CMDS.iteritems()):
+            self.comboBoxPresets.addItem(name, cmd)
+
         self.loadConf()
 
+    def __setPresetsCmd(self):
+    
+        #self.CaptureCommand.clear()
+        command = self.comboBoxPresets.itemData(self.comboBoxPresets.currentIndex(), QtCore.Qt.UserRole).toString()
+        self.CaptureCommand.setText(command)
+
     def loadConf(self):
 
         # Use conf from GApp.systconf['capture'] it it exist,
@@ -44,23 +57,13 @@ class UiConfig_PreferencesCapture(QtGui.QWidget, Ui_PreferencesCapture):
             self.conf = globals.GApp.systconf['capture']
         else:
             self.conf = systemCaptureConf()
-            
-        # Defaults capture terminal command
+
+        # Set default capture command
         if self.conf.cap_cmd == '':
-            if sys.platform.startswith('darwin'):
-                self.conf.cap_cmd = unicode("/usr/bin/open -a /Applications/Wireshark.app %c")
-            elif sys.platform.startswith('win'):
-                self.conf.cap_cmd = unicode("C:\Program Files\Wireshark\wireshark.exe %c")
-            else:
-                self.conf.cap_cmd = unicode("/usr/bin/wireshark %c")
+            self.conf.cap_cmd = CAPTURE_DEFAULT_CMD
 
         if self.conf.workdir == '':
-            if os.environ.has_key("TEMP"):
-                self.conf.workdir = unicode(os.environ["TEMP"])
-            elif os.environ.has_key("TMP"):
-                self.conf.workdir = unicode(os.environ["TMP"])
-            else:
-                self.conf.workdir = unicode('/tmp')
+            self.conf.workdir = CAPTURE_DEFAULT_WORKDIR
 
         # Push default values to GUI
         self.CaptureWorkingDirectory.setText(os.path.normpath(self.conf.workdir))
@@ -72,8 +75,8 @@ class UiConfig_PreferencesCapture(QtGui.QWidget, Ui_PreferencesCapture):
 
     def saveConf(self):
 
-        self.conf.workdir = unicode(self.CaptureWorkingDirectory.text())
-        self.conf.cap_cmd = unicode(self.CaptureCommand.text())
+        self.conf.workdir = unicode(self.CaptureWorkingDirectory.text(), 'utf-8', errors='replace')
+        self.conf.cap_cmd = unicode(self.CaptureCommand.text(), 'utf-8', errors='replace')
 
         if self.checkBoxStartCaptureCommand.checkState() == QtCore.Qt.Checked:
             self.conf.auto_start = True
