@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import sys, os
@@ -26,7 +26,9 @@ from GNS3.Config.Objects import systemDynamipsConf
 from GNS3.HypervisorManager import HypervisorManager
 from GNS3.Config.Config import ConfDB
 from GNS3.Utils import fileBrowser, translate, testOpenFile
+from GNS3.Config.Defaults import DYNAMIPS_DEFAULT_PATH, DYNAMIPS_DEFAULT_WORKDIR
 import GNS3.Globals as globals
+import subprocess as sub
 
 class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
 
@@ -37,7 +39,12 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
         self.connect(self.dynamips_path_browser, QtCore.SIGNAL('clicked()'), self.__setDynamipsPath)
         self.connect(self.dynamips_workdir_browser, QtCore.SIGNAL('clicked()'), self.__setDynamipsWorkdir)
         self.connect(self.pushButtonTestDynamips, QtCore.SIGNAL('clicked()'),self.__testDynamips)
-        self.comboBoxBinding.addItems(['localhost', QtNetwork.QHostInfo.localHostName()] + map(lambda addr: addr.toString(), QtNetwork.QNetworkInterface.allAddresses()))
+        #self.comboBoxBinding.addItems(['localhost', QtNetwork.QHostInfo.localHostName()] + map(lambda addr: addr.toString(), QtNetwork.QNetworkInterface.allAddresses()))
+        mylist = map(lambda addr: addr.toString(), QtNetwork.QNetworkInterface.allAddresses())
+        if mylist.__contains__('0:0:0:0:0:0:0:1'):
+            self.comboBoxBinding.addItems(['localhost', '::1', QtNetwork.QHostInfo.localHostName()] + mylist)
+        else:
+            self.comboBoxBinding.addItems(['localhost', QtNetwork.QHostInfo.localHostName()] + mylist)
         self.loadConf()
 
     def loadConf(self):
@@ -52,18 +59,13 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
         else:
             self.conf = systemDynamipsConf()
 
-        # Default path to dynamips executable
-        if self.conf.path == '' and sys.platform.startswith('win'):
-            self.conf.path = unicode('dynamips-wxp.exe')
-            
-        # Default path to working directory
+        # Set default path to dynamips executable
+        if self.conf.path == '':
+            self.conf.path = DYNAMIPS_DEFAULT_PATH
+
+        # Set default path to working directory
         if self.conf.workdir == '':
-            if os.environ.has_key("TEMP"):
-                self.conf.workdir = unicode(os.environ["TEMP"], errors='replace')
-            elif os.environ.has_key("TMP"):
-                self.conf.workdir = unicode(os.environ["TMP"], errors='replace')
-            else:
-                self.conf.workdir = unicode('/tmp', errors='replace')
+            self.conf.workdir = DYNAMIPS_DEFAULT_WORKDIR
 
         # Push default values to GUI
         self.dynamips_path.setText(os.path.normpath(self.conf.path))
@@ -83,6 +85,10 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
             self.checkBoxHypervisorManagerImport.setCheckState(QtCore.Qt.Checked)
         else:
             self.checkBoxHypervisorManagerImport.setCheckState(QtCore.Qt.Unchecked)
+        if self.conf.allocateHypervisorPerIOS == True:
+            self.checkBoxAllocatePerIOS.setCheckState(QtCore.Qt.Checked)
+        else:
+            self.checkBoxAllocatePerIOS.setCheckState(QtCore.Qt.Unchecked)
         if self.conf.ghosting == True:
             self.checkBoxGhosting.setCheckState(QtCore.Qt.Checked)
         else:
@@ -108,8 +114,8 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
         """ Save widget settings to syst. config
         """
         
-        self.conf.workdir = unicode(self.dynamips_workdir.text())
-        self.conf.path = unicode(self.dynamips_path.text())
+        self.conf.workdir = unicode(self.dynamips_workdir.text(), 'utf-8', errors='replace')
+        self.conf.path = unicode(self.dynamips_path.text(), 'utf-8', errors='replace')
         self.conf.port = self.dynamips_port.value()
         self.conf.baseUDP = self.dynamips_baseUDP.value()
         self.conf.baseConsole = self.dynamips_baseConsole.value()
@@ -124,6 +130,10 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
             self.conf.import_use_HypervisorManager = True
         else:
             self.conf.import_use_HypervisorManager = False
+        if self.checkBoxAllocatePerIOS.checkState() == QtCore.Qt.Checked:
+            self.conf.allocateHypervisorPerIOS = True
+        else:
+            self.conf.allocateHypervisorPerIOS = False
         if self.checkBoxGhosting.checkState() == QtCore.Qt.Checked:
             self.conf.ghosting = True
         else:
@@ -142,7 +152,7 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
             self.conf.mmap = False
         
         # update IOS images used by the hypervisor manager
-        binding = unicode(self.comboBoxBinding.currentText())
+        binding = unicode(self.comboBoxBinding.currentText(), 'utf-8', errors='replace')
         if self.conf.HypervisorManager_binding != binding:
             self.conf.HypervisorManager_binding = binding
             for name in globals.GApp.iosimages.keys():
@@ -163,7 +173,7 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
         if path is not None and path != '':
             # test if we can open it
             if not testOpenFile(path):
-                QtGui.QMessageBox.critical(globals.preferencesWindow, 'Dynamips path', unicode(translate("UiConfig_PreferencesDynamips", "Can't open file: %s")) % path)
+                QtGui.QMessageBox.critical(globals.preferencesWindow, 'Dynamips path', translate("UiConfig_PreferencesDynamips", "Can't open file: %s") % path)
                 return
 
             self.dynamips_path.clear()
@@ -199,11 +209,41 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
             if reply == QtGui.QMessageBox.No:
                 return
 
+        self.saveConf()
+        if globals.GApp.systconf['dynamips'].path:
+            if os.path.exists(globals.GApp.systconf['dynamips'].path) == False:
+                self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Dynamips path doesn't exist")  + '</font>')
+                return
+
+        try:
+            p = sub.Popen([globals.GApp.systconf['dynamips'].path, '--help'], stdout = sub.PIPE)
+            dynamips_stdout = p.communicate()
+        except OSError:
+            self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Failed to start Dynamips")  + '</font>')
+            return
+            
+        try:
+            if not dynamips_stdout[0].splitlines()[0].__contains__('version'):
+                self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Failed to determine version of Dynamips.")  + '</font>')
+                return
+            version_raw = dynamips_stdout[0].splitlines()[0].split('version')[1].lstrip()
+            version_1st = int(version_raw.split('.')[0])
+            version_2nd = int(version_raw.split('.')[1])
+            version_3rd = int(version_raw.split('.')[2].split('-')[0])
+            dynamips_ver = str(version_1st)+'.'+str(version_2nd)+'.'+str(version_3rd)+'-'+version_raw.split('.')[2].split('-')[1]
+        except:
+            self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Failed to determine version of Dynamips.")  + '</font>')
+            return            
+        
+        if version_2nd < 2 or version_3rd < 8:
+            self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Found Dynamips %s, which is not supported. Use 0.2.8+ instead.") % dynamips_ver + '</font>')
+            return
+
         self.saveConf()
         if globals.GApp.systconf['dynamips'].path:
             globals.GApp.workspace.clear()
             globals.GApp.HypervisorManager = HypervisorManager()
             if globals.GApp.HypervisorManager.preloadDynamips():
-                self.labelDynamipsStatus.setText('<font color="green">' + translate("UiConfig_PreferencesDynamips", "Dynamips successfully started")  + '</font>')
+                self.labelDynamipsStatus.setText('<font color="green">' + translate("UiConfig_PreferencesDynamips", "Dynamips %s successfully started") % dynamips_ver + '</font>')
             else:
                 self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Failed to start Dynamips")  + '</font>')
