@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007 GNS-3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,18 +16,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# http://www.gns3.net/contact
 #
 
-import sys
 import os
+import platform
+import shutil
 import GNS3.Globals as globals
 from PyQt4 import QtGui, QtCore
 from GNS3.Config.Objects import systemGeneralConf
 from GNS3.Ui.ConfigurationPages.Form_PreferencesGeneral import Ui_PreferencesGeneral
-from GNS3.Utils import translate, fileBrowser
+from GNS3.Utils import translate, fileBrowser, debug
+from GNS3.Config.Defaults import TERMINAL_PRESET_CMDS, TERMINAL_DEFAULT_CMD, PROJECT_DEFAULT_DIR, IOS_DEFAULT_DIR
 from GNS3.Config.Config import ConfDB
-from __main__ import VERSION_INTEGER
 
 class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
 
@@ -41,16 +42,22 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
             lang_name = lang[1]
             lang_displayText = u"%s (%s)" % (lang_name, lang_code)
             self.langsBox.addItem(lang_displayText)
+
+        if platform.system() == 'Darwin':
+            self.checkBoxBringConsoleToFront.setVisible(False)
         
         self.connect(self.pushButton_ClearConfiguration, QtCore.SIGNAL('clicked()'), self.__clearConfiguration)
         self.connect(self.ProjectPath_browser, QtCore.SIGNAL('clicked()'), self.__setProjectPath)
         self.connect(self.IOSPath_browser, QtCore.SIGNAL('clicked()'), self.__setIOSPath)
+        self.connect(self.pushButtonUseTerminalCommand, QtCore.SIGNAL('clicked()'), self.__setTerminalCmd)
+
+        for (name, cmd) in sorted(TERMINAL_PRESET_CMDS.iteritems()):
+            self.comboBoxPreconfigTerminalCommands.addItem(name, cmd)
+            
         self.loadConf()
 
     def loadConf(self):
 
-        import sys
-
         if globals.GApp.systconf.has_key('general'):
             self.conf = globals.GApp.systconf['general']
         else:
@@ -65,60 +72,32 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
                 self.langsBox.setCurrentIndex(idx)
             idx += 1
                 
-        # Defaults terminal command
+        # Slow start when starting every devices
+        self.slowStartAll.setValue(self.conf.slow_start)
+        
+        # Autosave
+        self.autoSave.setValue(self.conf.autosave)
+
+        # Set default terminal command
         if self.conf.term_cmd == '':
-            if sys.platform.startswith('darwin'):
-                self.conf.term_cmd = unicode("/usr/bin/osascript -e 'tell application \"terminal\" to do script with command \"telnet %h %p ; exit\"'")
-            elif sys.platform.startswith('win'):
-                if os.path.lexists('C:\\WINDOWS\\system32\\telnet.exe'):
-                    # check if telnet is there
-                    self.conf.term_cmd = unicode("start telnet %h %p")
-                else:
-                
-                    try:
-                        # try to detect Windows 64 bits
-                        import ctypes, sys
-                        i = ctypes.c_int()
-                        kernel32 = ctypes.windll.kernel32
-                        process = kernel32.GetCurrentProcess()
-                        kernel32.IsWow64Process(process, ctypes.byref(i))
-                        is64bit = (i.value != 0)
-                        if is64bit:
-                            # if 64 bits use Putty 32 bits
-                            self.conf.term_cmd = unicode('C:\Program Files (x86)\Putty\putty.exe -telnet %h %p')
-                        else:
-                            # else use the normal path
-                            self.conf.term_cmd = unicode('C:\Program Files\Putty\putty.exe -telnet %h %p')
-                    except:
-                        self.conf.term_cmd = unicode('C:\Program Files\Putty\putty.exe -telnet %h %p')
-                    self.conf.use_shell = False
-
-            else:
-                self.conf.term_cmd = unicode("xterm -T %d -e 'telnet %h %p' >/dev/null 2>&1 &")
+            self.conf.term_cmd = TERMINAL_DEFAULT_CMD
 
+        # Set default project dir
         if self.conf.project_path == '':
-            if os.environ.has_key("TEMP"):
-                self.conf.project_path = unicode(os.environ["TEMP"], errors='replace')
-            elif os.environ.has_key("TMP"):
-                self.conf.project_path = unicode(os.environ["TMP"],  errors='replace')
-            else:
-                self.conf.project_path = unicode('/tmp', errors='replace')
+            self.conf.project_path = PROJECT_DEFAULT_DIR
 
+        # Set default IOS image dir
         if self.conf.ios_path == '':
-            if os.environ.has_key("TEMP"):
-                self.conf.ios_path = unicode(os.environ["TEMP"], errors='replace')
-            elif os.environ.has_key("TMP"):
-                self.conf.ios_path = unicode(os.environ["TMP"],  errors='replace')
-            else:
-                self.conf.ios_path = unicode('/tmp', errors='replace')
+            self.conf.ios_path = IOS_DEFAULT_DIR
 
         self.lineEditTermCommand.setText(self.conf.term_cmd)
         self.ProjectPath.setText(os.path.normpath(self.conf.project_path))
         self.IOSPath.setText(os.path.normpath(self.conf.ios_path))
+
+        # GUI settings        
         self.workspaceWidth.setValue(self.conf.scene_width)
         self.workspaceHeight.setValue(self.conf.scene_height)
-            
-        # GUI settings
+        
         if self.conf.status_points == True:
             self.checkBoxShowStatusPoints.setCheckState(QtCore.Qt.Checked)
         else:
@@ -131,6 +110,10 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
             self.checkBoxUseShell.setCheckState(QtCore.Qt.Checked)
         else:
             self.checkBoxUseShell.setCheckState(QtCore.Qt.Unchecked)
+        if self.conf.bring_console_to_front == True:
+            self.checkBoxBringConsoleToFront.setCheckState(QtCore.Qt.Checked)
+        else:
+            self.checkBoxBringConsoleToFront.setCheckState(QtCore.Qt.Unchecked)
         if self.conf.project_startup == True:
             self.checkBoxProjectDialog.setCheckState(QtCore.Qt.Checked)
         else:
@@ -141,7 +124,20 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
         else:
             self.checkBoxDrawRectangle.setCheckState(QtCore.Qt.Unchecked)
             
-        self.labelConfigurationPath.setText(ConfDB().fileName())
+        if self.conf.relative_paths == True:
+            self.checkBoxRelativePaths.setCheckState(QtCore.Qt.Checked)
+        else:
+            self.checkBoxRelativePaths.setCheckState(QtCore.Qt.Unchecked)
+            
+        if self.conf.auto_check_for_update == True:
+            self.checkBoxCheckForUpdate.setCheckState(QtCore.Qt.Checked)
+        else:
+            self.checkBoxCheckForUpdate.setCheckState(QtCore.Qt.Unchecked)
+            
+        self.labelConfigurationPath.setText(os.path.normpath(unicode(ConfDB().fileName())))
+
+        # Delay between console starts
+        self.doubleSpinBoxConsoleDelay.setValue(self.conf.console_delay)
 
     def saveConf(self):
 
@@ -170,6 +166,10 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
             self.conf.use_shell = True
         else:
             self.conf.use_shell = False
+        if self.checkBoxBringConsoleToFront.checkState() == QtCore.Qt.Checked:
+            self.conf.bring_console_to_front = True
+        else:
+            self.conf.bring_console_to_front = False
         if self.checkBoxProjectDialog.checkState() == QtCore.Qt.Checked:
             self.conf.project_startup = True
         else:
@@ -180,12 +180,47 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
         else:
             self.conf.draw_selected_rectangle = False
 
-        self.conf.project_path = unicode(self.ProjectPath.text())
-        self.conf.ios_path = unicode(self.IOSPath.text())
-        self.conf.term_cmd = unicode(self.lineEditTermCommand.text())
+        if self.checkBoxRelativePaths.checkState() == QtCore.Qt.Checked:
+            self.conf.relative_paths = True
+        else:
+            self.conf.relative_paths = False
+            
+        if self.checkBoxCheckForUpdate.checkState() == QtCore.Qt.Checked:
+            self.conf.auto_check_for_update = True
+        else:
+            self.conf.auto_check_for_update = False
+            self.conf.last_check_for_update = 0
+
+        self.conf.project_path = unicode(self.ProjectPath.text(), 'utf-8', errors='replace')
+        self.conf.ios_path = unicode(self.IOSPath.text(), 'utf-8', errors='replace')
+        self.conf.term_cmd = unicode(self.lineEditTermCommand.text(), 'utf-8', errors='replace')
         self.conf.scene_width = self.workspaceWidth.value()
         self.conf.scene_height = self.workspaceHeight.value()
-        
+        self.conf.slow_start = self.slowStartAll.value()
+        self.conf.autosave = self.autoSave.value()
+        self.conf.console_delay = self.doubleSpinBoxConsoleDelay.value()
+
+        # Create project and image directories if they don't exist
+        if self.conf.project_path and not os.path.exists(self.conf.project_path):
+            try:
+                os.makedirs(self.conf.project_path)
+            except (OSError, IOError), e:
+                QtGui.QMessageBox.critical(globals.preferencesWindow, translate("UiConfig_PreferencesGeneral", "Project directory"),
+                                           translate("UiConfig_PreferencesGeneral", "Cannot create project directory: %s") % e.strerror)
+
+        if self.conf.ios_path and not os.path.exists(self.conf.ios_path):
+            try:
+                os.makedirs(self.conf.ios_path)
+            except (OSError, IOError), e:
+                QtGui.QMessageBox.critical(globals.preferencesWindow, translate("UiConfig_PreferencesGeneral", "Image directory"),
+                                           translate("UiConfig_PreferencesGeneral", "Cannot create image directory: %s") % e.strerror)
+                
+        if not os.path.exists(self.conf.ios_path + os.sep + 'baseconfig.txt'):
+            try:
+                shutil.copyfile('baseconfig.txt', self.conf.ios_path + os.sep + 'baseconfig.txt')
+            except (OSError, IOError), e:
+                debug("Warning: cannot copy baseconfig.txt to " + self.conf.ios_path + ": " + e.strerror)
+
         # Apply scene size
         globals.GApp.topology.setSceneRect(-(self.conf.scene_width / 2), -(self.conf.scene_height / 2), self.conf.scene_width, self.conf.scene_height)
 
@@ -202,14 +237,21 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
         
     def __setIOSPath(self):
     
-        fb = fileBrowser(translate('UiConfig_PreferencesGeneral', 'IOS Directory'), parent=globals.preferencesWindow)
+        fb = fileBrowser(translate('UiConfig_PreferencesGeneral', 'Image Directory'), parent=globals.preferencesWindow)
         path = fb.getDir()
 
         if path:
             self.IOSPath.setText(os.path.normpath(path))
+            
+    def __setTerminalCmd(self):
+    
+        self.lineEditTermCommand.clear()
+        command = self.comboBoxPreconfigTerminalCommands.itemData(self.comboBoxPreconfigTerminalCommands.currentIndex(), QtCore.Qt.UserRole).toString()
+        self.lineEditTermCommand.setText(command)
 
     def __clearConfiguration(self):
     
+        from __main__ import VERSION_INTEGER
         ConfDB().clear()
         c = ConfDB()
         c.set('GNS3/version', VERSION_INTEGER)
@@ -217,5 +259,6 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
         QtGui.QMessageBox.information(globals.preferencesWindow, translate("UiConfig_PreferencesGeneral", "Configuration file"),  
                                       translate("UiConfig_PreferencesGeneral", "Configuration file cleared, default settings will be applied after a restart"))
         globals.recordConfiguration = False
+        globals.preferencesWindow.close()
 
 
