@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,11 +16,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# http://www.gns3.net/contact
 #
 
-import os
+import os, sys, re
+import subprocess as sub
 import GNS3.Globals as globals
+import subprocess
 from PyQt4 import QtCore, QtGui
 
 class  Singleton(object):
@@ -37,7 +39,7 @@ def translate(context, text):
         context: string (classname)
         text: string (original text)
     """
-    return QtGui.QApplication.translate(context, text, None, QtGui.QApplication.UnicodeUTF8)
+    return unicode(QtGui.QApplication.translate(context, text, None, QtGui.QApplication.UnicodeUTF8))
 
 def testOpenFile(path,  flags='r'):
     """ returns True if the file can be openned
@@ -57,54 +59,69 @@ def debug(string):
 
         # Level 2, GNS3 debugs
         if globals.debugLevel >= 2:
-            print '* DEBUG: ' + str(string)
+            print '* DEBUG: ' + unicode(string)
             #globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
 
 def error(msg):
     """ Print out an error message
     """
 
-    print '*** Error:', str(msg)
+    print '*** Error:', unicode(msg)
 
-def relpath(target, base=os.curdir):
-    """
-    Return a relative path to the target from either the current dir or an optional base dir.
-    Base can be a directory specified either as absolute or relative to current dir.
-    Copied from Python CookBook
-    Author: R.Barran 30/08/2004
+def killAll(process_name):
+    """ Killall
     """
 
-    if not os.path.exists(target):
-        raise OSError, 'Target does not exist: '+target
-
-    if not os.path.isdir(base):
-        raise OSError, 'Base is not a directory or does not exist: '+base
-
-    base_list = (os.path.abspath(base)).split(os.sep)
-    target_list = (os.path.abspath(target)).split(os.sep)
-
-    # On the windows platform the target may be on a completely different drive from the base.
-    if os.name in ['nt','dos','os2'] and base_list[0] <> target_list[0]:
-        raise OSError, 'Target is on a different drive to base. Target: '+target_list[0].upper()+', base: '+base_list[0].upper()
-
-    # Starting from the filepath root, work out how much of the filepath is
-    # shared by base and target.
-    for i in range(min(len(base_list), len(target_list))):
-        if base_list[i] <> target_list[i]: break
+    if sys.platform.startswith('win'):
+        command = ['taskkill.exe', '/f', '/t', '/im']
     else:
-        # If we broke out of the loop, i is pointing to the first differing path elements.
-        # If we didn't break out of the loop, i is pointing to identical path elements.
-        # Increment i so that in all cases it points to the first differing path elements.
-        i+=1
+        command = ['killall', '-SIGKILL']
+    try:
+        subprocess.call(command + [process_name])
+        return True
+    except:
+        return False
+
+def getWindowsInterfaces():
+    """ Try to detect all available interfaces on Windows
+    """
+
+    try:
+        import _winreg
+    except:
+        pass
+
+    interfaces = []
+    dynamips = globals.GApp.systconf['dynamips']
+    if dynamips == '':
+        return []
+    try:
+        p = sub.Popen(dynamips.path + ' -e', stdout=sub.PIPE, stderr=sub.STDOUT)
+        outputlines = p.stdout.readlines()
+        p.wait()
+        for line in outputlines:
+            match = re.search(r"""^rpcap://\\Device\\NPF_({[a-fA-F0-9\-]*}).*""",  line.strip())
+            if match:
+                interface_name = u": "
+                try:
+                    reg_key = "SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}\\%s\\Connection" % match.group(1)
+                    key = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, reg_key, _winreg.KEY_READ)
+                    (value, typevalue) = _winreg.QueryValueEx(key, 'Name')
+                    _winreg.CloseKey(key)
+                    interface_name += unicode(value)
+                except:
+                    interface_name += u"unknown name"
+                    pass
+                interfaces.append(unicode(match.group(0)) + interface_name)
+    except:
+        return []
+    return interfaces
 
-    rel_list = [os.pardir] * (len(base_list)-i) + target_list[i:]
-    return os.path.join(*rel_list)
-    
 class fileBrowser(object):
     """ fileBrowser class
     """
 
-    def __init__(self, caption, directory = '.', filter = 'All files (*.*)', parent = None):
+    def __init__(self, caption, directory = '.', filter = 'All files (*)', parent = None):
 
         self.filedialog = QtGui.QFileDialog(parent)
         self.selected = QtCore.QString()
@@ -123,16 +140,6 @@ class fileBrowser(object):
             path = unicode(path)
         return ([path, str(self.selected)])
 
-    def getDir(self):
-        """ Get a directory from the file system
-        """
-
-        path = QtGui.QFileDialog.getExistingDirectory(self.filedialog,
-            self.caption, self.directory, QtGui.QFileDialog.ShowDirsOnly)
-        if path is not None:
-            path = unicode(path)
-        return (path)
-
     def getSaveFile(self):
         """ Save a file in the file system
         """
@@ -144,3 +151,12 @@ class fileBrowser(object):
             path = unicode(path)
         return ([path, str(self.selected)])
 
+    def getDir(self):
+        """ Get a directory from the file system
+        """
+
+        path = QtGui.QFileDialog.getExistingDirectory(self.filedialog,
+            self.caption, self.directory, QtGui.QFileDialog.ShowDirsOnly)
+        if path is not None:
+            path = unicode(path)
+        return (path)
